from rest_framework import mixins, viewsets
from rest_framework import authentication, permissions
from baykeshop.contrib.shop.models import BaykeShopOrdersComment
from .serializers import BaykeShopOrdersCommentSerializer



class BaykeShopOrdersCommentViewSet(mixins.CreateModelMixin, 
                                    viewsets.GenericViewSet):
    """订单评论"""
    queryset = BaykeShopOrdersComment.objects.all()
    serializer_class = BaykeShopOrdersCommentSerializer
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)

    


from django.db import models
from django.utils.translation import gettext_lazy as _

from .base import BaseModel


class BaseCategoryModel(BaseModel):
    """
    分类基类
    """
    name = models.CharField(max_length=50, verbose_name=_('名称'))
    parent = models.ForeignKey('self', blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('父级'))
    is_show = models.BooleanField(default=True, verbose_name=_('是否显示'))
    order = models.IntegerField(default=0, verbose_name=_('排序'))

    class Meta:
        abstract = True
        ordering = ['order']

    @classmethod
    def get_queryset(cls):
        return cls.objects.filter(is_show=True)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :cash.py
@说明    :收银台视图
@时间    :2024/12/03 10:49:25
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import TemplateView
from baykeshop.contrib.shop.models import BaykeShopGoodsSKU, BaykeShopCarts


class BaykeShopCashView(LoginRequiredMixin, TemplateView):
    """收银台视图"""
    template_name = 'baykeshop/shop/cash.html'
    login_url = 'member:login'
    extra_context = {
        'title': '收银台',
    }
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['has_carts'] = self.has_carts()
        context['skus'] = self.get_queryset()
        context['total'] = self.get_total_price()
        context['count'] = self.get_total_count()
        return context
    
    def has_carts(self):
        """判断购物车是购物车商品"""
        return 'skuids' in self.kwargs
    
    def get_queryset(self):
        """获取商品数据"""
        skuids = []
        if self.has_carts():
            skuids = [int(skuid) for skuid in self.kwargs.get('skuids').split(',')]
            queryset = BaykeShopCarts.objects.filter(sku_id__in=skuids, user=self.request.user)
            return queryset
        else:
            skuids = [int(self.kwargs.get('skuid'))]
            num = self.kwargs.get('num', 1)
            queryset = BaykeShopGoodsSKU.objects.filter(id__in=skuids).annotate(
                total_price = models.ExpressionWrapper(
                    models.F('price') * models.Value(num, output_field=models.IntegerField()),
                    output_field=models.DecimalField()
                ),
                quantity = models.Value(num, output_field=models.IntegerField()),
            )
            return queryset
    
    def get_total_price(self):
        total = sum([obj.total_price for obj in self.get_queryset()])
        return total
    
    def get_total_count(self):
        total = sum([obj.quantity for obj in self.get_queryset()])
        return total
    
    def get_login_url(self):
        messages.warning(self.request, _('请先登录后操作！'))
        return super().get_login_url()


from django.views.generic.detail import SingleObjectMixin
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib import messages
from django.views.generic.edit import ProcessFormView
from django.utils.translation import gettext_lazy as _
from django.http import JsonResponse, HttpResponseRedirect
from django.urls import reverse

from baykeshop.contrib.shop.models import BaykeShopOrders
from baykeshop.contrib.member.forms import BaykeShopOrdersCommentForm


class OrderStatusActionView(LoginRequiredMixin, SingleObjectMixin, ProcessFormView):
    """ 订单确认收货 """
    model = BaykeShopOrders
    slug_field = 'order_sn'
    slug_url_kwarg = 'order_sn'
    context_object_name = 'order'

    def get_queryset(self):
        return super().get_queryset().filter(user=self.request.user)
    
    def handle_no_permission(self):
        return JsonResponse({'code': 401, 'msg': '未登录，请登录后操作！'})
    
    def post(self, request, *args, **kwargs):
        order = self.get_object()
        order.status = BaykeShopOrders.OrderStatus.SIGNED
        order.save()
        messages.success(request, '确认收货成功！')
        return JsonResponse({'code': 200, 'msg': '确认收货成功！'})
    

class CommentActionView(OrderStatusActionView):
    """ 评论操作视图 """
    model = BaykeShopOrders
    slug_field = 'order_sn'
    slug_url_kwarg = 'order_sn'
    context_object_name = 'order'

    def post(self, request, *args, **kwargs):
        order = self.get_object()
        form = BaykeShopOrdersCommentForm(request.POST)
        # 判断订单状态
        if order.status != BaykeShopOrders.OrderStatus.SIGNED:
            return JsonResponse({'code': 400, 'msg': '订单状态不正确'})
        
        if form.is_valid():
            new_comment = form.save(commit=False)
            new_comment.order = order
            new_comment.user = request.user
            new_comment.save()
            # 更新订单状态
            order.status = BaykeShopOrders.OrderStatus.DONE
            order.is_comment = True
            order.save()
            messages.success(request, '评论成功！')
            return HttpResponseRedirect(reverse('member:orders-list'))
        return JsonResponse({'code': 400, 'msg': '评论失败！', 'data': form.errors})


# Generated by Django 4.2.17 on 2024-12-08 05:13

import baykeshop.db.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baykeshopuser',
            name='mobile',
            field=models.CharField(blank=True, max_length=11, null=True, unique=True, validators=[baykeshop.db.validators.validate_phone], verbose_name='手机号码'),
        ),
        migrations.AlterField(
            model_name='baykeshopuseraddress',
            name='phone',
            field=models.CharField(max_length=50, validators=[baykeshop.db.validators.validate_phone], verbose_name='手机'),
        ),
    ]



from django.db import models
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _
from django.template.loader import render_to_string
from baykeshop.db import BaseModel, BaseArticleModel, BaseCategoryModel
# Create your models here.


class BaykeArticleCategory(BaseCategoryModel):
    """ 文章分类 """

    class Meta:
        verbose_name = _('文章分类')
        verbose_name_plural = _('文章分类')
        ordering = ['-created_time']

    def __str__(self):
        return self.name


class BaykeArticleTags(BaseModel):
    """ 文章标签 """
    name = models.CharField(max_length=50, verbose_name=_('标签名称'))

    class Meta:
        verbose_name = _('文章标签')
        verbose_name_plural = _('文章标签')
        ordering = ['-created_time']

    def __str__(self):
        return self.name


class BaykeArticleContent(BaseArticleModel):
    """ 文章内容 """
    category = models.ForeignKey(
        BaykeArticleCategory,
        on_delete=models.SET_NULL, 
        verbose_name=_('文章分类'),
        blank=True,
        null=True
    )
    tags = models.ManyToManyField(
        BaykeArticleTags,
        blank=True,
        verbose_name=_('文章标签')
    )

    class Meta:
        verbose_name = _('文章内容')
        verbose_name_plural = _('文章内容')
        ordering = ['-created_time']

    def __str__(self):
        return self.title
    
    @property
    def description(self):
        desc = strip_tags(self.content)[:250]
        return desc
    
    @property
    def next_article(self):
        """ 下一篇文章 """
        try:
            article = self.get_next_by_created_time()
            return article
        except BaykeArticleContent.DoesNotExist:
            return None
    
    @property
    def prev_article(self):
        """ 上一篇文章 """
        try:
            article = self.get_previous_by_created_time()
            return article
        except BaykeArticleContent.DoesNotExist:
            return None
    

class BaykeSidebar(BaseModel):
    """ 侧边栏 """
    class ModuleChoices(models.TextChoices):
        """ 模板名称 """
        SEARCH = 'search', _('搜索')
        ARCHIVE = 'archive', _('归档')
        NAV = 'nav', _('导航栏')
        TAGS = 'tags', _('标签')
        CUSTOM = 'custom', _('自定义')

    name = models.CharField(max_length=50, verbose_name=_('名称'))
    icon = models.CharField(max_length=50, blank=True, default="mdi mdi-menu", verbose_name=_('图标'))
    module = models.CharField(
        max_length=255, 
        choices=ModuleChoices.choices,
        verbose_name=_('模块'),
        default=ModuleChoices.CUSTOM
    )
    content = models.TextField(
        verbose_name=_('自定义内容'), 
        blank=True, 
        default="", 
        help_text=_('自定义内容，仅在自定义模块下生效，支持html语法')
    )
    is_show = models.BooleanField(default=True, verbose_name=_('是否显示'))
    order = models.IntegerField(default=0, verbose_name=_('排序'))

    class Meta:
        verbose_name = _('侧边栏')
        verbose_name_plural = _('侧边栏')
        ordering = ['order']

    def __str__(self):
        return self.name

    def render(self):
        """ 渲染侧边栏 """
        context = {'sidebar': self}
        return render_to_string(f'baykeshop/sidebar/{self.module}.html', context)
    
    @property
    def navs(self):
        return BaykeArticleCategory.objects.filter(parent__isnull=True)
    
    @property
    def tags(self):
        tags_theme = ['bk-is-primary', 'bk-is-info', 'bk-is-success', 'bk-is-warning', 'bk-is-danger']
        return BaykeArticleTags.objects.annotate(
            count=models.Count('baykearticlecontent'),
            theme=models.Value(tags_theme, output_field=models.JSONField())
        )
    
    @property
    def archive(self):
        return BaykeArticleContent.objects.dates(field_name="created_time", kind="month")[:12]


import json

from django.template import Library
from django.urls import reverse

from baykeshop.contrib.shop.models import (
    BaykeShopCategory,
    BaykeShopBrand,
    BaykeShopGoods,
    BaykeShopCarts,
    BaykeShopOrders,
    BaykeShopOrdersComment,
)

register = Library()


@register.simple_tag
def paginator_range(page_obj, on_each_side=1):
    """分页页码数据"""
    return page_obj.paginator.get_elided_page_range(
        page_obj.number, on_each_side=on_each_side
    )


@register.inclusion_tag("baykeshop/tags/pagination.html")
def paginator_template(page_obj, request):
    """分页导模版"""
    params = request.GET.copy()
    params.pop("page", None)
    return {"urlencode": params.urlencode(), "page_obj": page_obj}


@register.inclusion_tag("baykeshop/tags/filters.html", takes_context=True)
def filters_template(context, request):
    """过滤器模板"""
    params = request.GET.copy()
    params.pop("page", None)
    params.pop("sort", None)
    params = params.urlencode()
    kwargs = context["view"].kwargs
    return {
        "request": request,
        "filter_params": params,
        "category_id": kwargs.get("pk", None),
        "object": context.get("object"),
        "brand_id": int(request.GET.get("brand_id", 0)),
    }


def __parent_category():
    return BaykeShopCategory.objects.filter(parent__isnull=True)


def __child_category(cate_id=None):
    try:
        cate = BaykeShopCategory.objects.get(id=cate_id)
        # 如果是子类
        if cate.parent:
            # 获取同类
            queryset = cate.parent.baykeshopcategory_set.all()
        else:
            # 获取子类
            queryset = cate.baykeshopcategory_set.all()
    except BaykeShopCategory.DoesNotExist:
        # 没有父级分类
        if not __parent_category().exists():
            return BaykeShopCategory.objects.none()
        # 有父级分类
        queryset = __parent_category().first().baykeshopcategory_set.all()
    return queryset


def __brand_queryset():
    return BaykeShopBrand.objects.all()


@register.simple_tag(name="parent_category_queryset")
def parent_category_queryset():
    return __parent_category()


@register.simple_tag(name="child_category_queryset")
def child_category_queryset(cate_id=None):
    return __child_category(cate_id)


@register.simple_tag(name="brand_queryset")
def brand_queryset():
    return __brand_queryset()


@register.inclusion_tag("baykeshop/tags/sort.html")
def sort_template(request, filter_params: str, sort_field="created_time", **kwargs):
    """排序模板"""
    sort = request.GET.get("sort", "")
    is_asc = sort and sort.startswith("-")
    is_desc = sort and not sort.startswith("-")
    if sort_field == sort:
        sort_field = f"-{sort}"

    return {
        "filter_params": filter_params,
        "request": request,
        "sort_field": sort_field,
        "is_active": sort_field.lstrip("-") == sort.lstrip("-"),
        "is_asc": is_asc,
        "is_desc": is_desc,
        **kwargs,
    }


@register.simple_tag
def has_many_sku(spu: BaykeShopGoods):
    """判断商品是否是多规格
    Args:
        spu (BaykeShopGoods): 商品对象
    Returns:
        bool: 是否多规格
    """
    return spu.has_many_sku()


def _group_by_spec(data):
    """
    将数据按规格名称分类

    :param data: 包含规格数据的列表
    :return: 按规格名称分类的字典
    """
    grouped_data = {}

    for item in data:
        spec_name = item["parent__name"]
        spec_value = item["name"]

        if spec_name not in grouped_data:
            grouped_data[spec_name] = []

        grouped_data[spec_name].append(spec_value)
        grouped_data[spec_name] = list(set(grouped_data[spec_name]))
    return grouped_data


@register.inclusion_tag("baykeshop/tags/sku.html")
def sku_template(spu):
    """商品规格模板"""
    sku_queryset = spu.baykeshopgoodssku_set.order_by("price")
    # 所有的商品规格数据
    specs_data = []

    # 获取sku对应的商品数据
    skus_data = {}
    for sku in sku_queryset:
        specs = json.loads(sku.specs)
        specs_data.extend(specs)
        key = ",".join([spec["name"] for spec in specs]) if specs else "default"
        skus_data[key] = {
            "id": sku.id,
            "spu_id": sku.goods.id,
            "price": str(sku.price),
            "stock": sku.stock,
            "line_price": str(sku.line_price),
            "sales": sku.sales,
        }
    specs = _group_by_spec(specs_data)
    return {
        "skus_data": skus_data,
        "specs_data": specs,
    }


@register.simple_tag
def carts_count(user):
    """获取购物车数量"""
    if not user.is_authenticated:
        return 0
    count = BaykeShopCarts.objects.filter(user=user).count()
    return count


def _insert_update_url(data, request):
    """插入修改地址url
    方便修改地址后能快速跳转回原地址
    """
    for item in data:
        item["update_url"] = (
            f"{reverse('member:address-update', kwargs={'pk': item['id']})}?next={request.get_full_path()}"
        )
    return data


@register.inclusion_tag("baykeshop/tags/address.html", takes_context=True)
def address_template(context, user):
    address_list = user.baykeshopuseraddress_set.values()
    request = context["request"]
    return {
        "address_list": _insert_update_url(list(address_list), request),
        "request": request,
    }


@register.inclusion_tag("baykeshop/tags/paytype.html")
def paytype_template():
    choices = BaykeShopOrders.PayType.choices
    _types = [{"value": item[0], "label": item[1]} for item in choices]
    return {"pay_types": _types}


@register.filter
def json_loads(value):
    """json loads"""
    return json.loads(value)


@register.simple_tag
def paydata(skus, has_carts=False):
    """json dumps"""
    print
    results = []
    for sku in skus:
        item = {}
        if has_carts:
            item["sku"] = sku.sku.id
        else:
            item["sku"] = sku.id
        item["price"] = str(sku.price)
        item["quantity"] = sku.quantity
        results.append(item)
    return results


@register.simple_tag
def score_avg(spu):
    """商品平均评分"""
    return BaykeShopOrdersComment.get_score_avg(spu)


@register.simple_tag
def rate(spu):
    """商品评论率"""
    return BaykeShopOrdersComment.get_spu_comment_avg_score(spu)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ArticleConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'baykeshop.contrib.article'
    verbose_name = _('文章管理')
    verbose_name_plural = _('文章管理')



from django.urls import reverse_lazy
from django.views.generic import DetailView, ListView
from django.views.generic.edit import FormView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import JsonResponse
from django.utils.translation import gettext_lazy as _
from django.utils import timezone

from baykeshop.contrib.shop.models import BaykeShopOrders
from baykeshop.contrib.member.forms import BaykeShopOrdersCommentForm


class BaykeShopOrdersListView(LoginRequiredMixin, ListView):
    """ 订单列表 """
    template_name = 'baykeshop/member/order_list.html'
    login_url = reverse_lazy('member:login')
    model = BaykeShopOrders
    paginate_by = 10
    ordering = '-created_time'

    def get_queryset(self):
        status = self.request.GET.get('status')
        queryset = super().get_queryset().filter(user=self.request.user)
        if status:
            queryset = queryset.filter(status=status)
        return queryset
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _('我的订单')
        context['comment_form'] = BaykeShopOrdersCommentForm()
        return context
    

class BaykeShopOrdersDetailView(LoginRequiredMixin, DetailView):
    """ 订单详情 """
    template_name = 'baykeshop/member/order_detail.html'
    login_url = reverse_lazy('member:login')
    model = BaykeShopOrders
    slug_field = 'order_sn'
    slug_url_kwarg = 'order_sn'
    context_object_name = 'order'

    def get_queryset(self):
        return super().get_queryset().filter(user=self.request.user)
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _('订单详情')
        return context
    


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :urls.py
@说明    :接口路由总文件
@时间    :2024/12/01 09:51:50
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''
from django.urls import path
from rest_framework import routers

from baykeshop.api.carts import views as carts_views
from baykeshop.api.orders import views as orders_views
from baykeshop.api.pay import views as pay_views
from baykeshop.api.comments import views as comments_views
from baykeshop.api.upload import views as upload_views


router = routers.DefaultRouter()
# 路由命名空间
app_name = 'baykeshop_api'
# 购物车
router.register('carts', carts_views.BaykeShopCartsViewSet, basename='carts')

"""
支付订单,处理支付逻辑
根据选择的支付方式返回对应的支付地址
    @url: {% url 'baykeshop_api:pay-detail' order_sn=订单编号 %}
    @method: post
"""
router.register('pay', pay_views.BaykeShopOrdersPayView, basename='pay')
# 订单评论
router.register('comments', comments_views.BaykeShopOrdersCommentViewSet, basename='comments')

"""
创建订单及删除订单
1.创建订单 
    @url: {% url 'baykeshop_api:orders-list' %} 
    @method: post
2.删除订单 
    @url: {% url 'baykeshop_api:orders-detail' order_sn=订单编号 %} 
    @method: delete
"""
router.register('orders', orders_views.BaykeShopOrdersViewSet, basename='orders')

urlpatterns = [
    path('upload/image/', upload_views.UploadImageView.as_view(), name='upload-image'),
    *router.urls
]


from .goods import (
    BaykeShopGoodsListView, BaykeShopCategoryListView, 
    BaykeShopGoodsDetailView, BaykeShopSearchView
)
from .carts import BaykeShopCartsListView
from .cash import BaykeShopCashView
from .pay import BaykeShopOrdersPayView, AlipayCallbackView
from .public import BaykeShopIndexView



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

# Create your models here.
from baykeshop.db import BaseModel, BaseUserModel
from baykeshop.db import validators


User = get_user_model()


class BaykeShopUser(BaseUserModel):
    """用户表"""

    user = models.OneToOneField(User, on_delete=models.CASCADE, verbose_name=_("用户"))

    def __str__(self):
        return self.user.username


class BaykeShopUserAddress(BaseModel):
    """用户地址"""

    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_("用户"))
    name = models.CharField(max_length=50, verbose_name=_("收货人"))
    province = models.CharField(max_length=50, verbose_name=_("省"))
    city = models.CharField(max_length=50, verbose_name=_("市"))
    district = models.CharField(max_length=50, verbose_name=_("区"))
    address = models.CharField(max_length=255, verbose_name=_("详细地址"))
    phone = models.CharField(
        max_length=50, verbose_name=_("手机"), validators=[validators.validate_phone]
    )
    is_default = models.BooleanField(default=False, verbose_name=_("是否默认"))

    class Meta:
        verbose_name = _("用户地址")
        verbose_name_plural = verbose_name
        ordering = ["-created_time"]
        indexes = [
            models.Index(fields=["user", "is_default"], name="user_is_default_idx"),
        ]

    def __str__(self):
        return self.name

    def get_full_address(self):
        return "{province}{city}{district}{address}".format(
            province=self.province,
            city=self.city,
            district=self.district,
            address=self.address,
        )



from typing import Any
from django.db.models.query import QuerySet
from django.contrib.auth import get_user_model
from django.views.generic import ListView, MonthArchiveView
from django.views.generic.detail import DetailView, SingleObjectMixin
from django.utils.translation import gettext_lazy as _
# Create your views here.
from baykeshop.contrib.system.models import Visit
from .models import BaykeArticleCategory, BaykeArticleTags, BaykeArticleContent

# Create your views here.

User = get_user_model()


class BaykeArticleContentListView(ListView):
    """ 全部文章 """
    model = BaykeArticleContent
    template_name = 'baykeshop/article/list.html'
    paginate_by = 10
    ordering = ['-created_time']

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _('全部文章')
        return context


class BaykeArticleSearchView(BaykeArticleContentListView):
    """ 搜索 """
    def get_queryset(self):
        query = self.request.GET.get('q')
        query = query.strip() if query else ''
        if len(query) > 100:
            raise ValueError(_('搜索内容过长'))
        return super().get_queryset().filter(title__icontains=query)
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.request.GET.get('q', '')
        return context


class BaykeArticleCategoryListView(SingleObjectMixin, ListView):
    """ 分类文章 """
    model = BaykeArticleContent
    template_name = 'baykeshop/article/list.html'
    paginate_by = 10
    ordering = ['-created_time']

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BaykeArticleCategory.objects.all())
        return super().get(request, *args, **kwargs)

    def get_queryset(self):
        return self.object.baykearticlecontent_set.all()
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.name
        return context


class BaykeArticleTagsListView(SingleObjectMixin, ListView):
    """ 标签文章 """
    model = BaykeArticleContent
    template_name = 'baykeshop/article/list.html'
    paginate_by = 10
    ordering = ['-created_time']

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BaykeArticleTags.objects.all())
        return super().get(request, *args, **kwargs)
    
    def get_queryset(self):
        return self.object.baykearticlecontent_set.all()
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.name
        return context


class BaykeArticleContentDetailView(DetailView):
    """ 文章详情 """
    model = BaykeArticleContent
    template_name = 'baykeshop/article/detail.html'
    context_object_name = 'article'

    def get_object(self, queryset=None):
        obj = super().get_object(queryset)
        Visit.objects.create_pv_uv(self.request, obj)
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.title
        return context
    

class BaykeArticleArchiveView(MonthArchiveView):
    """ 归档 """
    queryset = BaykeArticleContent.objects.all()
    date_field = 'created_time'
    template_name = 'baykeshop/article/list.html'
    allow_empty = True
    make_object_list = True
    month_format = '%m'
    paginate_by = 10

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _('文章归档')
        return context
    

class BaykeArticleUserListView(SingleObjectMixin, ListView):
    """ 用户文章 """
    model = BaykeArticleContent
    template_name = 'baykeshop/article/list.html'
    paginate_by = 10
    ordering = ['-created_time']

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=User.objects.all())
        return super().get(request, *args, **kwargs)

    def get_queryset(self):
        return self.object.baykearticlecontent_set.all()
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = f"{self.object.username} 的文章"
        return context


from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.utils.html import format_html

# Register your models here.

from .models import BaykeShopUser

User = get_user_model()
admin.site.unregister(User)


class BaykeShopUserAdmin(admin.StackedInline):
    model = BaykeShopUser
    extra = 1


@admin.register(User)
class BaseUserAdmin(UserAdmin):
    list_display = ("username", "nickname", "avatar", "email", "mobile", "is_staff")
    list_display_links = ("username", "nickname")
    inlines = (BaykeShopUserAdmin,)
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (_("Personal info"), {"fields": ("email",)}),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "password1", "password2"),
            },
        ),
    )


    @admin.display(description="头像")
    def avatar(self, obj):
        if not obj.baykeshopuser.avatar:
            return None
        return format_html(
            '<img src="{}" width="40px" height="40px" />', obj.baykeshopuser.avatar.url
        )

    @admin.display(description="昵称")
    def nickname(self, obj):
        return obj.baykeshopuser.nickname

    @admin.display(description="手机")
    def mobile(self, obj):
        return obj.baykeshopuser.mobile



# Generated by Django 4.2.17 on 2024-12-07 14:11

from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeShopUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('nickname', models.CharField(blank=True, max_length=50, null=True, verbose_name='昵称')),
                ('avatar', models.ImageField(blank=True, null=True, upload_to='avatar', verbose_name='头像')),
                ('gender', models.CharField(choices=[('male', '男'), ('female', '女'), ('unknown', '未知')], default='male', max_length=10, verbose_name='性别')),
                ('birthday', models.DateField(blank=True, null=True, verbose_name='生日')),
                ('mobile', models.CharField(blank=True, max_length=11, null=True, unique=True, verbose_name='手机号码')),
                ('qq', models.CharField(blank=True, max_length=20, null=True, verbose_name='QQ')),
                ('wechat', models.CharField(blank=True, max_length=50, null=True, verbose_name='微信')),
                ('description', models.TextField(blank=True, null=True, verbose_name='简介')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '用户信息',
                'verbose_name_plural': '用户信息',
                'ordering': ['-created_time'],
                'abstract': False,
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopUserAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='收货人')),
                ('province', models.CharField(max_length=50, verbose_name='省')),
                ('city', models.CharField(max_length=50, verbose_name='市')),
                ('district', models.CharField(max_length=50, verbose_name='区')),
                ('address', models.CharField(max_length=255, verbose_name='详细地址')),
                ('phone', models.CharField(max_length=50, verbose_name='手机')),
                ('is_default', models.BooleanField(default=False, verbose_name='是否默认')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '用户地址',
                'verbose_name_plural': '用户地址',
                'ordering': ['-created_time'],
                'indexes': [models.Index(fields=['user', 'is_default'], name='user_is_default_idx')],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from baykeshop.contrib.shop.models import BaykeShopOrdersComment, BaykeShopOrders


class BaykeShopOrdersCommentSerializer(serializers.ModelSerializer):
    """ 订单评论序列化器 """
    class Meta:
        model = BaykeShopOrdersComment
        fields = ('order', 'content', 'score')

    def validate_order(self, value):
        """ 验证订单 """
        request = self.context.get('request')
        if not (value.order.user == request.user):
            messages.error(request, _('订单与当前用户不匹配'))
            raise serializers.ValidationError(_('订单与当前用户不匹配'))

        if value.is_comment:
            messages.error(request, _('订单已评论, 请勿重复评论'))
            raise serializers.ValidationError(_('订单已评论, 请勿重复评论'))
        return value
    
    def create(self, validated_data):
        """ 创建评论 """
        instance = super().create(validated_data)
        instance.order.is_comment = True
        instance.order.status = BaykeShopOrders.OrderStatus.DONE
        instance.order.save()
        messages.success(self.context.get('request'), _('评论成功'))
        return instance
    


from django.urls import path

from . import views

app_name = 'member'

urlpatterns = [
    path('login/', views.BaykeShopUserLoginView.as_view(), name='login'),
    path('logout/', views.BaykeShopUserLogoutView.as_view(), name='logout'),
    path('register/', views.BaykeShopUserRegisterView.as_view(), name='register'),
    path('profile/', views.BaykeShopUserProfileView.as_view(), name='profile'),
    path('profile/<int:pk>/update/', views.BaykeShopUserProfileUpdateView.as_view(), name='profile-update'),
    path('address/', views.BaykeShopUserAddressListView.as_view(), name='address-list'),
    path('password/', views.BaykeShopUserPasswordView.as_view(), name='password'),
    path('address/create/', views.BaykeShopUserAddressCreateView.as_view(), name='address-create'),
    path('address/<int:pk>/update/', views.BaykeShopUserAddressUpdateView.as_view(), name='address-update'),
    path('address/<int:pk>/delete/', views.BaykeShopUserAddressDeleteView.as_view(), name='address-delete'),
    path('orders/', views.BaykeShopOrdersListView.as_view(), name='orders-list'),
    path('orders/<slug:order_sn>/', views.BaykeShopOrdersDetailView.as_view(), name='orders-detail'),
    # 订单相关操作
    path('orders/<slug:order_sn>/comment/', views.CommentActionView.as_view(), name='orders-comment'),
    path('orders/<slug:order_sn>/receipt/', views.OrderStatusActionView.as_view(), name='orders-receipt'),
]


from django.views.generic import TemplateView
from django.utils.translation import gettext_lazy as _

from baykeshop.contrib.shop.models import BaykeShopCategory, BaykeShopGoods


class BaykeShopIndexView(TemplateView):
    template_name = 'baykeshop/index.html'

    def get_floors(self):
        """ 获取楼层 """
        category_list = BaykeShopCategory.objects.filter(is_floor=True, parent__isnull=True)
        for category in category_list:
            sub_category_list = category.baykeshopcategory_set.all()
            spu_list = BaykeShopGoods.objects.filter(category__in=sub_category_list)
            category.spu_list = spu_list
        return category_list
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['floors'] = self.get_floors()
        context['title'] = _('首页')
        return context
        


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :serializers.py
@说明    :创建订单序列化器
@时间    :2024/12/03 22:41:22
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.db.models.signals import post_save
from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from django.urls import reverse

from rest_framework import serializers

from baykeshop.contrib.shop.models import (
    BaykeShopOrdersGoods, BaykeShopOrders, BaykeShopGoodsImages,
    BaykeShopCarts
)

 
class BaykeShopOrdersGoodsSerializer(serializers.ModelSerializer):
    class Meta:
        model = BaykeShopOrdersGoods
        fields = ('sku', 'quantity')


class BaykeShopOrdersCreateSerializer(serializers.ModelSerializer):
    """订单创建序列化器"""
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())
    baykeshopordersgoods_set = BaykeShopOrdersGoodsSerializer(many=True)
    """ 创建订单的几种方式
        carts: 购物车，这里会查找购物车去清理数据
        default: 直接创建不做任何处理
    """
    source = serializers.ChoiceField(
        choices=('carts', 'default'), 
        help_text=_('订单来源'), 
        required=False,
        write_only=True,
        default='default'
    )
    pay_url = serializers.CharField(required=False, read_only=True, help_text=_('支付地址'))

    class Meta:
        model = BaykeShopOrders
        fields = (
            'user', 'baykeshopordersgoods_set', 
            'receiver', 'phone', 'address', 'source',
            'pay_url'
        )
    
    def validate(self, attrs):
        """验证数据"""
        baykeshopordersgoods_set = attrs.get('baykeshopordersgoods_set')
        if not baykeshopordersgoods_set:
            raise serializers.ValidationError(_('请选择商品'))
        for item in baykeshopordersgoods_set:
            if int(item['quantity']) <= 0:
                raise serializers.ValidationError(_('商品数量必须大于0'))
            sku = item.get('sku')
            if sku.stock < int(item['quantity']):
                raise serializers.ValidationError(_('商品库存不足'))
        return attrs

    def create(self, validated_data):
        """创建订单"""
        source = validated_data.pop('source')
        baykeshopordersgoods_set = validated_data.pop('baykeshopordersgoods_set')
        pay_price = sum([item['sku'].price * item['quantity'] for item in baykeshopordersgoods_set])
        orders = BaykeShopOrders.objects.create(pay_price=pay_price, **validated_data)
        created_objects = BaykeShopOrdersGoods.objects.bulk_create(
            [BaykeShopOrdersGoods(orders=orders, **self.goods_format(item)) for item in baykeshopordersgoods_set]
        )
        # 手动触发post_save信号
        for obj in created_objects:
            post_save.send(sender=BaykeShopOrdersGoods, instance=obj, created=True)
        # 清理购物车数据
        if source == 'carts':
            skus = [item['sku'] for item in baykeshopordersgoods_set]
            BaykeShopCarts.objects.filter(user=validated_data['user'], sku__in=skus).delete()
        orders.pay_url = reverse('shop:orders-pay', kwargs={'order_sn': orders.order_sn})
        messages.success(self.context['request'], _('订单创建成功, 请尽快支付, 否则订单会自动取消'))
        return orders
    
    def get_image(self, sku):
        images = BaykeShopGoodsImages.objects.filter(goods=sku.goods)
        if images.exists(): 
            return images.first().image
        return ''

    def goods_format(self, item):
        """商品格式化"""
        item['price'] = item['sku'].price
        item['sku_sn'] = item['sku'].sku_sn
        item['name'] = item['sku'].goods.name
        item['image'] = self.get_image(item['sku'])
        item['specs'] = item['sku'].specs
        item['detail'] = item['sku'].goods.detail
        return item


import json

from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView, DetailView

from baykeshop.contrib.shop.models import BaykeShopCarts

User = get_user_model()


class BaykeShopCartsListView(LoginRequiredMixin, ListView):
    """ 购物车列表 """
    template_name = 'baykeshop/shop/carts.html'
    context_object_name = 'carts_list'
    extra_context = {
        'title': _('购物车'),
    }

    def get_queryset(self):
        queryset = BaykeShopCarts.objects.filter(user=self.request.user).order_by('-created_time')
        queryset = list(queryset.values(
            'id', 'total_price', 'name', 'specs', 'image_url', 'sku_id',
            'sku__price', 'sku__stock', 'sku__sales', 'quantity'
        ))
        for item in queryset:
            item['specs'] = json.loads(item['specs'])
            item['total_price'] = round(item['total_price'], 2)
        return queryset




from django.db import models
from django.db.models.functions import TruncDate
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.utils.translation import gettext_lazy as _


User = get_user_model()


class BaseManager(models.Manager):
    """
    自定义管理器
    """

    def get_queryset(self):
        return super().get_queryset()
    
    def delete(self):
        return super().update(is_delete=True)
    
    def hard_delete(self):
        return super().delete()
    
    def all_delete(self):
        return super().update(is_delete=False)
    
    def all_hard_delete(self):
        return super().all().delete()
    
    def truncdate_queryset(self):
        queryset = self.get_queryset().annotate(date=TruncDate('created_time'))
        return queryset


class BaseModel(models.Model):
    """
    基础模型
    """
    site = models.ForeignKey(Site, on_delete=models.CASCADE, verbose_name=_('站点'), editable=False, blank=True, null=True)
    created_time = models.DateTimeField(auto_now_add=True, verbose_name=_('创建时间'))
    updated_time = models.DateTimeField(auto_now=True, verbose_name=_('更新时间'))
    is_delete = models.BooleanField(default=False, editable=False, verbose_name=_('是否删除'))

    objects = BaseManager()
    current_site = CurrentSiteManager()

    class Meta:
        abstract = True
        ordering = ['-created_time']

    def save(self, *args, **kwargs):
        self.site = Site.objects.get_current()
        super().save(*args, **kwargs)



from datetime import timedelta
from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone
from django.conf import settings
from rest_framework import serializers

from baykeshop.contrib.shop.models import BaykeShopOrders
from baykeshop.payment import AliPay


class BaykeShopOrdersPaySerializer(serializers.ModelSerializer):
    """ 处理支付逻辑
    根据选择的支付方式返回对应的支付地址
    """
    pay_url = serializers.SerializerMethodField()
    
    class Meta:
        model = BaykeShopOrders
        fields = ('pay_type', 'pay_url')

    def validate_pay_type(self, value):
        if value == BaykeShopOrders.PayType.WECHATPAY:
            messages.error(self.context['request'], _('暂不支持微信支付'))
            raise serializers.ValidationError(_('暂不支持微信支付'))
        if self.instance.status != BaykeShopOrders.OrderStatus.UNPAID:
            messages.error(self.context['request'], _('订单状态错误'))
            raise serializers.ValidationError(_('该状态下的订单无法支付, 请联系客服'))
        return value

    def update(self, instance, validated_data):
        # 判断订单创建时间是否在一个小时之内,超时则不能再支付了,这是一种简便的处理方式
        # 后续可以考虑使用消息队列来处理超时订单，这里目前仅做了简单的阻止，库存并未回归
        if (timezone.now() - instance.created_time) > timedelta(hours=1):
            instance.status = BaykeShopOrders.OrderStatus.EXPIRED
            instance.save()
            messages.error(self.context['request'], _('订单已超时, 请重新下单'))
            raise serializers.ValidationError(_('订单已过期'))
        
        pay_type = validated_data.get('pay_type')
        if pay_type == BaykeShopOrders.PayType.ALIPAY:
            instance.pay_type = BaykeShopOrders.PayType.ALIPAY
        elif pay_type == BaykeShopOrders.PayType.CASH:
            instance.pay_type = BaykeShopOrders.PayType.CASH
            instance.pay_sn = instance.order_sn
            instance.is_verify = True
            instance.status = BaykeShopOrders.OrderStatus.VERIFY
        instance.save()
        return instance

    def get_pay_url(self, instance):
        """ 获取支付链接 """
        if instance.pay_type == BaykeShopOrders.PayType.CASH:
            messages.success(self.context['request'], _('订单已提交，核销中...'))
            return reverse('member:orders-detail', kwargs={'order_sn': instance.order_sn})
        
        # 支付宝回调地址
        callback_url=self.context['request'].build_absolute_uri(reverse('shop:alipay-callback'))
        alipay = AliPay(return_url=callback_url, notify_url=callback_url, debug=settings.DEBUG)
        alipay_url = alipay.trade_page_pay(instance)
        return alipay_url
    


import re
import json

from django.core import validators
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

def is_bool(value):
    """
    验证值是否为布尔值
    :param value: 值
    :return: 布尔值
    """
    if value.lower() in ['true', 'false']:
        return True
    return False

def is_dict(value):
    """
    验证值是否为字典
    :param value: 值
    :return: 布尔值
    """
    if all(re.match(r'^\s*\w+\s*:\s*.+\s*$', line) for line in value.splitlines()):
        return True
    return False

def is_list(value):
    """
    验证值是否为列表
    :param value: 值
    :return: 布尔值
    """

    if all(re.match(r'^\s*.+\s*$', line) for line in value.splitlines()):
        return True
    return False

def is_json(value):
    """
    验证值是否为JSON对象
    :param value: 值
    :return: 布尔值
    """
    try:
        json.loads(value)
        return True
    except json.JSONDecodeError:
        return False

def validate_dict_value(value):
    # 布尔值验证
    if is_bool(value):
        return True
        
    # 键值对验证
    if is_dict(value):
        return True
        
    # 列表验证
    if is_list(value):
        return True
        
    # JSON 对象验证
    if is_json(value):
        return True
    
    raise ValidationError(
        _("%(value)s is not valid"),
        params={"value": value},
        code='invalid',
    )
    



from django import forms
from django.template import Library

register = Library()

@register.filter
def is_input(field):
    """判断是否为输入框"""
    return isinstance(field.field, forms.CharField)

@register.filter
def is_select(field):
    """判断是否为下拉框"""
    return isinstance(field.field, forms.TypedChoiceField)

@register.filter
def is_image(field):
    """判断是否为图片上传框"""
    return isinstance(field.field, forms.ImageField)

@register.filter
def is_boolean(field):
    """判断是否为状态选择框"""
    return isinstance(field.field, forms.BooleanField)

@register.filter
def add_class(field, css_class):
    """为表单项添加css类"""
    if 'class' in field.field.widget.attrs:
        field.field.widget.attrs['class'] = ' '.join([field.field.widget.attrs['class'], css_class])
    else:
        field.field.widget.attrs['class'] = css_class
    return field

def change_template(field, template_name):
    """修改表单项的模板"""
    field.field.widget.template_name = template_name
    return field


from django.views.generic import ListView
from django.views.generic.detail import SingleObjectMixin, DetailView
from django.utils.translation import gettext_lazy as _
from django.core.paginator import Paginator

from baykeshop.contrib.shop.models import (
    BaykeShopCategory, BaykeShopGoods,
    BaykeShopOrdersComment
)


class BaykeShopGoodsListView(ListView):
    """商品列表"""
    template_name = 'baykeshop/shop/list.html'
    paginate_by = 20
    model = BaykeShopGoods
    ordering = ('-created_time',)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = "商品列表"
        return context
    
    def get_queryset(self):
        queryset = super().get_queryset()
        return self.filter_queryset(queryset)

    def filter_queryset(self, queryset):
        brand_id = self.request.GET.get("brand_id")
        sort = self.request.GET.get("sort", "-created_time")
        if brand_id:
            queryset = queryset.filter(brand_id=brand_id)
        if sort:
            queryset = queryset.order_by(sort)
        return queryset


class BaykeShopCategoryListView(SingleObjectMixin, BaykeShopGoodsListView):
    """商品分类列表"""
    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BaykeShopCategory.objects.all())
        return super().get(request, *args, **kwargs)
    
    def get_queryset(self):
        # 顶级分类返回所有子分类的商品
        queryset = BaykeShopGoods.objects.filter(category__id=self.object.id)
        if self.object.parent is None:
            baykeshopcategory_set = self.object.baykeshopcategory_set.all()
            queryset = BaykeShopGoods.objects.filter(category__in=baykeshopcategory_set)
            return self.filter_queryset(queryset)
        return self.filter_queryset(queryset)
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.name
        return context
    

class BaykeShopGoodsDetailView(DetailView):
    """商品详情"""
    model = BaykeShopGoods
    template_name = 'baykeshop/shop/detail.html'
    context_object_name = 'spu'
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.get_object().name
        context['images'] = self.get_images()
        context['recommends'] = self.get_recommend_queryset()
        context['comments'] = self.get_comments()
        context['score_avg'] = self.get_score_avg()
        context['like_score'] = self.get_like_score()
        context['comments_count'] = self.get_comments_count()
        return context
    
    def get_images(self):
        """ 获取商品图片 """
        queryset = self.get_object().baykeshopgoodsimages_set.order_by('order')
        return queryset
    
    def get_recommend_queryset(self):
        """ 获取同类别推荐的商品，按销量排序，排除当前商品 """
        cates = self.get_object().category.all()
        queryset = BaykeShopGoods.objects.filter(
            is_recommend=True, 
            category__in=cates
        ).exclude(
            id=self.get_object().id
        ).order_by('-sales')
        return queryset[:5]

    def get_comments(self):
        queryset = BaykeShopOrdersComment.get_spu_queryset(self.get_object())
        paginator = Paginator(queryset, 20)
        page_obj = paginator.get_page(self.request.GET.get('page', 1))
        return page_obj
    
    def get_score_avg(self):
        """ 获取商品平均评分 """
        return BaykeShopOrdersComment.get_score_avg(self.get_object())
    
    def get_like_score(self):
        """ 好评率 """
        return BaykeShopOrdersComment.get_spu_comment_avg_score(self.get_object())
    
    def get_comments_count(self):
        """ 获取商品评论数 """
        return BaykeShopOrdersComment.get_comment_count(self.get_object())



class BaykeShopSearchView(BaykeShopGoodsListView):
    """商品搜索"""
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = "商品搜索"
        return context
    
    def get_queryset(self):
        queryset = super().get_queryset()
        keyword = self.request.GET.get("keyword")
        if keyword:
            queryset =queryset.filter(name__icontains=keyword)
            return self.filter_queryset(queryset)
        return queryset


from django.db import models

from baykeshop.forms.widgets import RichTextWidget


class RichTextField(models.TextField):
    
    def formfield(self, **kwargs):
        kwargs['widget'] = RichTextWidget
        return super().formfield(**kwargs)
    


from django.contrib import admin
from django.contrib.sites.models import Site
from django.contrib.sites.admin import SiteAdmin
from django.utils.translation import gettext_lazy as _
from baykeshop.sites import admin as bayke_admin

# Register your models here.
from baykeshop.conf import bayke_settings
from .models import *

admin.site.unregister(Site)

@admin.register(Site)
class BaykeSiteAdmin(SiteAdmin):
    list_display = (
        "domain",
        "name",
    )
    search_fields = ("domain", "name")
    fieldsets = ((None, {"fields": ("domain", "name")}),)

    def has_add_permission(self, request):
        return False


@admin.register(BaykeDictModel)
class BaykeDictModelAdmin(bayke_admin.ModelAdmin):
    list_display = ("key", "name", "created_time")
    search_fields = ("key", "name", "value")
    fieldsets = ((None, {"fields": ("key", "name", "value")}),)

    def get_readonly_fields(self, request, obj=None):
        # 判断是否是系统内置字典
        if hasattr(bayke_settings, obj.key):
            return [
                "key",
            ]
        return super().get_readonly_fields(request, obj)

    def has_delete_permission(self, request, obj=None):
        # 判断是否是系统内置字典
        if obj and hasattr(bayke_settings, obj.key):
            self.message_user(request, _(f"{obj.key}为系统内置字典不允许删除"), "ERROR")
            return False
        return super().has_delete_permission(request, obj)


@admin.register(BaykeBanners)
class BaykeBannersAdmin(bayke_admin.ModelAdmin):
    list_display = ("title", "image", "url", "is_show", "order", "created_time")
    search_fields = ("title", "url")
    fieldsets = ((None, {"fields": ("title", "image", "url", "is_show", "order")}),)



from django.core import validators
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from baykeshop.conf import bayke_settings

def validate_phone(value):
    # 中国区手机号验证
    validators.RegexValidator(
        bayke_settings.REGEX_PHONE, 
        _("手机号码格式有误"), 
        "invalid"
    )(value)


def validate_image_size(value):
    """ 对图片大小进行验证 """
    if value.size > bayke_settings.MAX_IMAGE_SIZE:
        raise ValidationError(
            _("图片大小超过限制"), 
            code='invalid'
        )


from django.db import models
from django.utils.translation import gettext_lazy as _
from baykeshop.db import BaseModel


class BaykeBanners(BaseModel):
    """
    轮播图
    """
    title = models.CharField(max_length=50, verbose_name=_('标题'))
    image = models.ImageField(upload_to='banners', verbose_name=_('图片'))
    url = models.CharField(max_length=255, blank=True, default='', verbose_name=_('链接'))
    order = models.IntegerField(default=0, verbose_name=_('排序'))
    is_show = models.BooleanField(default=True, verbose_name=_('是否显示'))

    class Meta:
        verbose_name = _('轮播图')
        verbose_name_plural = _('轮播图')
        ordering = ('-order',)
    
    def __str__(self):
        return self.title


from django.contrib.auth import get_user_model
from django.contrib.auth.views import LoginView, LogoutView
from django.contrib.messages.views import SuccessMessageMixin
from django.contrib import messages
from django.views.generic import FormView
from django.utils.translation import gettext_lazy as _
from django.urls import reverse_lazy

from baykeshop.contrib.member.models import BaykeShopUser
from baykeshop.contrib.member.forms import LoginForm, RegisterForm

User = get_user_model()


class BaykeShopUserLoginView(SuccessMessageMixin, LoginView):
    """ 登录页面 """
    template_name = 'baykeshop/member/login.html'
    # redirect_authenticated_user = True
    redirect_field_name = 'next'
    next_page = reverse_lazy('shop:list')
    form_class = LoginForm
    extra_context = {
        'title': _('登录'),
    }
    success_message = _('登录成功')


class BaykeShopUserRegisterView(FormView):
    """ 注册页面 """
    template_name = 'baykeshop/member/register.html'
    form_class = RegisterForm
    success_url = reverse_lazy('member:login')
    extra_context = {
        'title': _('注册'),
    }

    def form_valid(self, form):
        """ 注册成功 """
        user = form.save()
        BaykeShopUser.objects.create(user=user, nickname=user.username)
        messages.success(self.request, _('注册成功'))
        return super().form_valid(form)
    

class BaykeShopUserLogoutView(LogoutView):
    """ 登出页面 """
    next_page = reverse_lazy('member:login')
    
    def get_success_url(self):
        messages.success(self.request, _('登出成功'))
        return super().get_success_url()
    



from django.db import models
from django.utils.translation import gettext_lazy as _

from baykeshop.db import (
    BaseModel,
    BaseGoodsModel,
    BaseGoodsSKUModel,
    BaseCategoryModel,
    BaseCartsModel,
)
from .managers import (
    BaykeShopGoodsManager,
    BaykeShopCartsManager,
    BaykeShopGoodsSKUManager,
)


class BaykeShopCategory(BaseCategoryModel):
    """商品分类"""

    icon = models.CharField(
        max_length=50, blank=True, default="", verbose_name=_("图标")
    )
    is_floor = models.BooleanField(default=False, verbose_name=_("是否楼层"))
    is_nav = models.BooleanField(default=False, verbose_name=_("是否导航"))

    class Meta:
        verbose_name = _("商品分类")
        verbose_name_plural = _("商品分类")
        ordering = ["order"]

    def __str__(self):
        return self.name


class BaykeShopBrand(BaseModel):
    """商品品牌"""

    name = models.CharField(max_length=50, verbose_name=_("品牌名称"))
    image = models.ImageField(
        upload_to="brand", blank=True, null=True, verbose_name=_("品牌图片")
    )
    order = models.IntegerField(default=0, verbose_name=_("排序"))
    description = models.TextField(blank=True, null=True, verbose_name=_("品牌介绍"))

    class Meta:
        verbose_name = _("商品品牌")
        verbose_name_plural = verbose_name
        ordering = ["order"]

    def __str__(self):
        return self.name


class BaykeShopGoods(BaseGoodsModel):
    """商品"""

    category = models.ManyToManyField(
        BaykeShopCategory, blank=True, verbose_name=_("商品分类")
    )
    brand = models.ForeignKey(
        BaykeShopBrand,
        on_delete=models.SET_NULL,
        verbose_name=_("商品品牌"),
        blank=True,
        null=True,
    )
    # 推荐
    is_recommend = models.BooleanField(default=False, verbose_name=_("商品推荐"))

    objects = BaykeShopGoodsManager()

    class Meta:
        verbose_name = _("商品")
        verbose_name_plural = _("商品")
        ordering = ["-created_time"]

    def __str__(self):
        return self.name

    def has_many_sku(self):
        return self.baykeshopgoodssku_set.count() > 1


class BaykeShopGoodsSKU(BaseGoodsSKUModel):
    """商品SKU"""

    goods = models.ForeignKey(
        BaykeShopGoods, on_delete=models.CASCADE, verbose_name=_("商品")
    )

    objects = BaykeShopGoodsSKUManager()

    class Meta:
        verbose_name = _("商品SKU")
        verbose_name_plural = _("商品SKU")
        ordering = ["-created_time"]

    def __str__(self):
        return self.goods.name


class BaykeShopCarts(BaseCartsModel):
    """购物车"""

    sku = models.ForeignKey(
        BaykeShopGoodsSKU, on_delete=models.CASCADE, verbose_name=_("商品")
    )

    objects = BaykeShopCartsManager()

    class Meta:
        verbose_name = _("购物车")
        verbose_name_plural = _("购物车")
        ordering = ["-created_time"]
        constraints = [
            models.UniqueConstraint(fields=["user", "sku"], name="unique_carts")
        ]

    def __str__(self):
        return f"{self.user.username} - {self.sku.goods.name}"


class BaykeShopSpec(BaseCategoryModel):
    """商品规格"""

    class Meta:
        verbose_name = _("规格模版")
        verbose_name_plural = _("规格模版")
        ordering = ["-created_time"]

    def __str__(self):
        if not self.parent:
            return self.name
        return f"{self.parent.name}:{self.name}"


class BaykeShopGoodsImages(BaseModel):
    """商品图片"""

    goods = models.ForeignKey(
        BaykeShopGoods, on_delete=models.CASCADE, verbose_name=_("商品")
    )
    image = models.ImageField(
        upload_to="goods/images",
        verbose_name=_("商品图片"),
        help_text="_(建议尺寸: 800*800，排在第一个的图片会作为商品主图)",
    )
    order = models.IntegerField(default=0, verbose_name=_("排序"))

    class Meta:
        verbose_name = _("商品图片")
        verbose_name_plural = _("商品图片")
        ordering = ["order"]

    def __str__(self):
        return f"{self.goods.name} - {self.image.url}"



DEFAULTS = {
    # 站点基本设置
    'SITE_ID': 1,
    'SITE_TITLE': 'BaykeShop',
    'SITE_HEADER': 'baykeShop开源商城系统',
    'INDEX_TITLE': '欢迎使用BaykeShop商城系统',
    'DESCRIPTION': '基于Django构建的开源商城系统',
    'KEYWORDS': 'baykeShop,商城系统,Django商城系统,Django商城',
    'COPYRIGHT': 'BaykeShop © 2024 All rights reserved.',
    'ICP': '陕ICP备19000001号',

    # 邮箱基本配置
    'EMAIL_HOST': 'smtp.qq.com',
    'EMAIL_HOST_USER': '',
    'EMAIL_HOST_PASSWORD': '',
    'EMAIL_PORT': 465,
    'EMAIL_USE_SSL': True,
    'EMAIL_USE_TLS': False,
    'EMAIL_BACKEND': 'django.core.mail.backends.smtp.EmailBackend',
    'DEFAULT_FROM_EMAIL': '',
    'EMAIL_SUBJECT_PREFIX': '[baykeShop]',

    # 支付宝配置
    'ALIPAY_APPID': '2021000122666025',
    'ALIPAY_PRIVATE_KEY': 'MIIEogIBAAKCAQEAm3zZGqRtgBpbAY1f4eaHqwCycWOf1hvRbp/xfnb5HeIzyE5s2y+/Hbfl1c32ZvXczUCwo1UqI8LS+FUn4q0Knn+RBoQqrFKCJpfLR3IsYbBA02aXPt3H8QKIEBtDsEAeOmCj1/zuVzO9Wi81FTzKe9nkeSV4OV75M715oCwyfZg1vbESA0wQXXoE7B9t5Q2tyba2RNpHhhs8Blto/Q+2oayxUwHcOsrTZVLrIZeI2Rayt02WokaWx11ENUYbgY0eV12PgXRRbqYxsoIBJE14q49a9nrN12q8Vq1DvhCy9Ma4j/7ZTZEiN5oxha+DxRlk5KJ4y7IswViMwc/BRjC0bwIDAQABAoIBABTXsn6Ixgji/4oAunacoVkB2emrMhTQW82ZcRTDu/cOiGR0ZtEm4FaKfSug1UPN91AoOMVcOC7dAcy6FYOkMIU4kmcuNk4tLTchJ+Wm314k8zujR+jsb83VyglmrjclEShAJhZg2E4mKF67b0EuifH6Lk8y/5+pJHyUDIa1BfuhwystbzTeNCEAOQQ1CF/wPnOul8v9k1xXwxpvFDYUxBQGLPXsdPyqJixzGviuOgZA7zjROUHi14VWUbYnCOtvJsezTtr9BWm9evMYNDsCMOjozovM0uiZkQAJk0TgtBjW3xAzUa/XQLb3AlcKvUjGhu5RLcxQpLD7X75eUgjDkLECgYEAyyMX3SiqQkpfyJe4TJBUPCD3ddwpdAGa9Xh9Q6jGO1tAI7ytLfbNYp+8MPAR8QsQO0hW5F4PaANHfosGuP11AvO0HwLYDZI67LNXN1+cSE7Jj3iwOqPXMxxP2UgY3xSzfceD4VVJlqN8UraXY7iWVnn7ConeRzhysFtW1StOZLcCgYEAw/NfSv33tiTtICNjEA0qjmxO5pSnK6QdcmewM5MHAdo3uL3VT1JwlGSwLxUiFiTz61UwvgHFqxMPiN0xVkNmZsfB19GE2n7hgHws/k13Ow3WzLoLElRYetCgSNk4wOsIa9qQxOG59s9AP1IyBYuKjdLi/Z1JggBfU64gjmpkJgkCgYAFp1Ris18JuUgxLQfUGbA+fCxm+1msAN9vNYtG9suNm1yNohKv1M6ikA+MzSGys7wf6kOA151WJ8E1pGjrfSJkkqPQuSOKPSemrtcek2qmGItDkvhUqXqz0XXndjo6+ziDp6nj+5uwtZwgMTTFcb29ameyQ2QMV2XZwrEH/cyQwQKBgAHP2Eu7OXUntkGb0iit1nzmp3weV8VhiRwrtUdkzqXHvMgqiTRokL3uiGqHCX+m69J7QS4gmhFQ5c3gLbqceO/uXO5/83iJv9AOVeQsFzQmlpviBIQw33iTQUgrKgkuoy9MIBaR98Q+elubiPbgKXHSR2MM90cKddcyPv7SIQbxAoGAQLpY5hRKQ6Pnz8T27eO900F4iYDmIKbodUtiKPrEiZ/PN1bmxwrZ5m81t6ukuLZ2kra0ukeepwxzmLOGl3ZDKeKVD0VyUuegdNfOTBEnnEn3nz/Yz9HC6DoLVp/oop0ROwiMrxBg8EpYbG2Qgru9DA5KaWiiJb0EO2U7fO//SNE=',
    'ALIPAY_PUBLIC_KEY': 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAnGm4s+aI1LsCwL9i67GSqEICxqfkyzydk6TIA57M9++EHBUAsXHa5mpgUIQwlYKetzsdPnYK8rS7Pkn6RFogF639zoaRBIpCu8K/lazUB2PqykLdV+XH8DqtH3k6lz1hFRAOHDIn3wVqIUOC0H/G4TFsp8Cd/cgLYAFr12Fgm20/OW8GZNnhfZmmcbHc4el9CqWuEt1xRQLAgLiaDjTZ5RrgSwHem2p1kYKjcs0jw1M+IyKIZK0k6s/KwwqsSlG28ysP94nBI1v3vSIL0e25rvv5irYXi78hfmmO8+sBdYxBzkaF7tndTctTxTYMcnk/+1jKijahDW/72zTI0AwSxwIDAQAB',
    

    # 手机号验证规则
    'REGEX_PHONE' : r'^1[3-9]\d{9}$',
    # 上传图片大小
    'MAX_IMAGE_SIZE': 2 * 1024 * 1024,
    # 富文本编辑配置
    'QUILL_CONFIGS': {
        'CSS': {
            'all': ('https://cdn.jsdelivr.net/npm/quill@2.0.2/dist/quill.snow.css',),
        },
        'JS': ('https://cdn.jsdelivr.net/npm/quill@2.0.2/dist/quill.js',),
        'DEFAULT': {
            'modules': {
                'toolbar': [
                    ['bold', 'italic', 'underline', 'strike'],        # toggled buttons
                    ['blockquote', 'code-block'],
                    ['link', 'image', 'video', 'formula'],

                    [{ 'header': 1 }, { 'header': 2 }],               # custom button values
                    [{ 'list': 'ordered'}, { 'list': 'bullet' }, { 'list': 'check' }],
                    [{ 'script': 'sub'}, { 'script': 'super' }],      # superscript/subscript
                    [{ 'indent': '-1'}, { 'indent': '+1' }],          # outdent/indent
                    [{ 'direction': 'rtl' }],                         # text direction

                    [{ 'size': ['small', False, 'large', 'huge'] }],  # custom dropdown
                    [{ 'header': [1, 2, 3, 4, 5, 6, False] }],

                    [{ 'color': [] }, { 'background': [] }],          # dropdown with defaults from theme
                    [{ 'font': [] }],
                    [{ 'align': [] }],

                    ['clean']                                         # remove formatting button
                ]
            },
            'theme': 'snow',
        }
    }
}

IMPORT_STRINGS = [
    'EMAIL_BACKEND'
]

REMOVED_SETTINGS = []


# Generated by Django 4.2.17 on 2024-12-09 08:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baykeshoporders',
            name='is_verify',
            field=models.BooleanField(default=False, verbose_name='核销订单'),
        ),
    ]



from django.urls import path

from . import views

app_name = 'article'

urlpatterns = [
    # 文章列表
    path('', views.BaykeArticleContentListView.as_view(), name='list'),
    # 文章详情
    path('<int:pk>/', views.BaykeArticleContentDetailView.as_view(), name='detail'),
    # 分类文章列表
    path('category/<int:pk>/', views.BaykeArticleCategoryListView.as_view(), name='category'),
    # 标签文章列表
    path('tags/<int:pk>/', views.BaykeArticleTagsListView.as_view(), name='tags'),
    # 搜索文章列表
    path('search/', views.BaykeArticleSearchView.as_view(), name='search'),
    # 归档文章列表
    path('archive/<int:year>/<int:month>/', views.BaykeArticleArchiveView.as_view(), name='archive'),
    # 用户文章列表
    path('user/<int:pk>/', views.BaykeArticleUserListView.as_view(), name='user'),
]





import random
from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.utils import timezone

from .base import BaseModel, BaseManager

User = get_user_model()


class BaykeShopOrdersGoodsManager(BaseManager):
    """订单商品管理器"""
    def get_queryset(self):
        return super().get_queryset().select_related('sku').alias(
            total_price=models.ExpressionWrapper(
                models.F('quantity') * models.F('price'),
                output_field=models.DecimalField()
            )
        ).annotate(
            total_price=models.F('total_price')
        )


class BaseOrdersModel(BaseModel):
    """ 订单基类 """

    class OrderStatus(models.IntegerChoices):
        UNPAID = 0, _('未支付')
        # 已支付
        PAID = 1, _('待发货')
        # 已发货
        SHIPPED = 2, _('待收货')
        # 已签收
        SIGNED = 3, _('待评价')
        # 评价完成
        DONE = 4, _('已完成')
        # 已过期（长时间未支付）
        EXPIRED = 5, _('已取消')
        # 已退款
        REFUNDED = 6, _('已退款')
        # 待核销
        VERIFY = 7, _('待核销')
    

    class PayType(models.IntegerChoices):
        ALIPAY = 0, _('支付宝')
        WECHATPAY = 1, _('微信支付')
        CASH = 2, _('货到付款')    
        
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('用户'))
    order_sn = models.CharField(max_length=50, verbose_name=_('订单号'))
    # total_price = models.DecimalField(max_digits=10, decimal_places=2, verbose_name=_('总价'))
    status = models.IntegerField(choices=OrderStatus.choices, default=OrderStatus.UNPAID, verbose_name=_('订单状态'))
    pay_price = models.DecimalField(max_digits=10, decimal_places=2, verbose_name=_('实际支付金额'))
    # 收货人
    receiver = models.CharField(max_length=50, verbose_name=_('收货人'), blank=True, default='')
    phone = models.CharField(max_length=11, verbose_name=_('手机号码'), blank=True, default='')
    address = models.CharField(max_length=255, verbose_name=_('收货地址'), blank=True, default='')
    pay_type = models.IntegerField(choices=PayType.choices, default=PayType.ALIPAY, verbose_name=_('支付方式'))
    # 支付流水号
    pay_sn = models.CharField(max_length=125, verbose_name=_('支付流水号'), blank=True, default='')
    pay_time = models.DateTimeField(blank=True, null=True, verbose_name=_('支付时间'))
    # 是否核销订单
    is_verify = models.BooleanField(default=False, verbose_name=_('核销订单'))
    verify_time = models.DateTimeField(blank=True, null=True, verbose_name=_('核销时间'))
    is_comment = models.BooleanField(default=False, verbose_name=_('是否评价'))

    class Meta:
        abstract = True
        constraints = [
            models.UniqueConstraint(fields=['user', 'order_sn'], name='unique_order_sn')
        ]

    def get_order_sn(self):
        return '{}{}{}'.format(
            timezone.now().strftime('%Y%m%d%H%M%S'), 
            self.user.id, 
            random.randint(10, 99)
        )

    def save(self, *args, **kwargs):
        if not self.order_sn:
            self.order_sn = self.get_order_sn()
        super().save(*args, **kwargs)


class BaseOrdersGoodsModel(BaseModel):
    """ 订单商品 """
    price = models.DecimalField(max_digits=10, decimal_places=2, verbose_name=_('价格'))
    quantity = models.IntegerField(default=1, verbose_name=_('数量'))
    specs = models.JSONField(
        verbose_name=_('规格'), 
        default=list,
        help_text=_('规格数据'),
        blank=True
    )
    # 商品SKU数据锁定状态
    sku_sn = models.CharField(max_length=50, verbose_name=_('商品编码'), blank=True, default='')
    name = models.CharField(max_length=50, verbose_name=_('商品名称'), blank=True, default='')
    detail = models.TextField(blank=True, null=True, verbose_name=_('商品详情'))
    image = models.ImageField(upload_to='goods', blank=True, null=True, verbose_name=_('商品主图'))

    objects = BaykeShopOrdersGoodsManager()

    class Meta:
        abstract = True


from django.contrib import admin
from baykeshop.forms import ModelForm
from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.db.analysis import  UserAnalysisService, OrderAnalysisService, VisitAnalysisService
from baykeshop.contrib.shop.models import BaykeShopOrders
from .echarts import orders_chart, users_chart, user_pie_chart


class TabularInline(admin.TabularInline):
    '''Tabular Inline View for '''
    form = ModelForm


class StackedInline(admin.StackedInline):
    '''Stacked Inline View for '''
    form = ModelForm


class ModelAdmin(admin.ModelAdmin):
    """自定义ModelAdmin"""
    form = ModelForm


class AdminSite(admin.AdminSite):
    """ 自定义AdminSite """

    index_template = 'baykeshop/admin/index.html'

    @property
    def site_header(self):
        return BaykeDictModel.get_key_value('SITE_HEADER')
    
    @property
    def site_title(self):
        return BaykeDictModel.get_key_value('SITE_TITLE')
    
    @property
    def index_title(self):
        return BaykeDictModel.get_key_value('INDEX_TITLE')

    def index(self, request, extra_context=None):
        extra_context = extra_context or {}
        # 用户分析
        user_analysis = UserAnalysisService(request)
        user_data = user_analysis.get_data('%m-%d')
        # 订单分析
        order_analysis = OrderAnalysisService(model=BaykeShopOrders)
        # 订单销售分析
        order_data = order_analysis.get_data('%m-%d')
        sales_data = order_analysis.get_sales_data('%m-%d')
        # 访问分析
        visit_analysis = VisitAnalysisService()
        visit_data = visit_analysis.get_data('%m-%d')
        # orders_chart()
        
        extra_context = {
            "order_data": order_data,
            "sales_data": sales_data,
            "user_data": user_data,
            "visit_data": visit_data,
            "orders_chart": orders_chart(),
            "users_chart": users_chart(),
            "user_pie_chart": user_pie_chart(),
        }
        return super().index(request, extra_context)
    

admin_site = AdminSite(name="baykeshop")




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :views.py
@说明    :上传图片视图
@时间    :2024/12/07 17:41:53
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''
from django.conf import settings
from django.core.files.storage import default_storage, FileSystemStorage

from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from rest_framework.authentication import SessionAuthentication


from .serializers import UploadImageSerializer


class CsrfExemptSessionAuthentication(SessionAuthentication):
    """ 关闭csrf验证 """
    def enforce_csrf(self, request):
        return


class UploadImageView(GenericAPIView):
    serializer_class = UploadImageSerializer
    authentication_classes = (CsrfExemptSessionAuthentication,)
    permission_classes = (IsAuthenticated,)

    def post(self, request, *args, **kwargs):
        """ 上传图片 """
        serializer = self.get_serializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        image = serializer.validated_data['file']
        # 上传到指定目录
        storeage = FileSystemStorage(
            location=settings.MEDIA_ROOT / 'uploads',
            base_url=settings.MEDIA_URL + 'uploads/'
        )
        file_name = storeage.save(image.name, image)
        url = storeage.url(file_name)
        return Response({'location': url})



# Generated by Django 4.2.17 on 2024-12-07 14:11

import baykeshop.contrib.system.validators
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeDictModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('key', models.SlugField(verbose_name='键')),
                ('value', models.TextField(help_text='\n            布尔值：true/false; \n            键值对：key1:value1一行一对;\n            list:value1,value2一行一个值;\n            json:{"key1":"value1"}\n        ', validators=[baykeshop.contrib.system.validators.validate_dict_value], verbose_name='值')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '字典管理',
                'verbose_name_plural': '字典管理',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeBanners',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('title', models.CharField(max_length=50, verbose_name='标题')),
                ('image', models.ImageField(upload_to='banners', verbose_name='图片')),
                ('url', models.CharField(blank=True, default='', max_length=255, verbose_name='链接')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '轮播图',
                'verbose_name_plural': '轮播图',
                'ordering': ('-order',),
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='Visit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('object_id', models.PositiveBigIntegerField(verbose_name='对象ID')),
                ('ip', models.GenericIPAddressField(default='127.0.0.1', verbose_name='IP')),
                ('pv', models.PositiveBigIntegerField(verbose_name='PV')),
                ('uv', models.PositiveBigIntegerField(verbose_name='UV')),
                ('date', models.DateField(verbose_name='日期')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='内容类型')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '分析数据',
                'verbose_name_plural': '分析数据',
                'ordering': ['-date'],
                'indexes': [models.Index(fields=['date'], name='system_visi_date_8fe15a_idx'), models.Index(fields=['content_type', 'object_id'], name='system_visi_content_0cf1e0_idx'), models.Index(fields=['content_type', 'object_id', 'date'], name='system_visi_content_8e96af_idx')],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddConstraint(
            model_name='baykedictmodel',
            constraint=models.UniqueConstraint(fields=('key', 'site'), name='unique_key_site'),
        ),
    ]



# Generated by Django 4.2.17 on 2024-12-07 14:11

import baykeshop.db.fields
from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeShopBrand',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='品牌名称')),
                ('image', models.ImageField(blank=True, null=True, upload_to='brand', verbose_name='品牌图片')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('description', models.TextField(blank=True, null=True, verbose_name='品牌介绍')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '商品品牌',
                'verbose_name_plural': '商品品牌',
                'ordering': ['order'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('icon', models.CharField(blank=True, default='', max_length=50, verbose_name='图标')),
                ('is_floor', models.BooleanField(default=False, verbose_name='是否楼层')),
                ('is_nav', models.BooleanField(default=False, verbose_name='是否导航')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopcategory', verbose_name='父级')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '商品分类',
                'verbose_name_plural': '商品分类',
                'ordering': ['order'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopGoods',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='商品名称')),
                ('keywords', models.CharField(blank=True, max_length=255, null=True, verbose_name='商品关键字')),
                ('description', models.TextField(blank=True, null=True, verbose_name='商品描述')),
                ('detail', baykeshop.db.fields.RichTextField(blank=True, null=True, verbose_name='商品详情')),
                ('status', models.PositiveSmallIntegerField(choices=[(1, '上架'), (2, '下架')], default=1, verbose_name='商品状态')),
                ('goods_type', models.PositiveSmallIntegerField(choices=[(1, '普通商品'), (2, '虚拟商品')], default=1, verbose_name='商品类型')),
                ('is_recommend', models.BooleanField(default=False, verbose_name='商品推荐')),
                ('brand', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.baykeshopbrand', verbose_name='商品品牌')),
                ('category', models.ManyToManyField(blank=True, to='shop.baykeshopcategory', verbose_name='商品分类')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '商品',
                'verbose_name_plural': '商品',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopGoodsSKU',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('specs', models.JSONField(blank=True, default=list, help_text='规格数据', verbose_name='规格')),
                ('price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='价格')),
                ('sku_sn', models.CharField(blank=True, default='', max_length=50, verbose_name='商品编码')),
                ('line_price', models.DecimalField(blank=True, decimal_places=2, default='', help_text='商品价格划线价，不参与价位计算筛选, 仅供前端显示使用', max_digits=10, verbose_name='划线价')),
                ('stock', models.PositiveSmallIntegerField(default=0, verbose_name='库存')),
                ('sales', models.PositiveSmallIntegerField(default=0, verbose_name='销量')),
                ('goods', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopgoods', verbose_name='商品')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '商品SKU',
                'verbose_name_plural': '商品SKU',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopOrders',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('order_sn', models.CharField(max_length=50, verbose_name='订单号')),
                ('status', models.IntegerField(choices=[(0, '未支付'), (1, '待发货'), (2, '待收货'), (3, '待评价'), (4, '已完成'), (5, '已取消'), (6, '已退款'), (7, '待核销')], default=0, verbose_name='订单状态')),
                ('pay_price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='实际支付金额')),
                ('receiver', models.CharField(blank=True, default='', max_length=50, verbose_name='收货人')),
                ('phone', models.CharField(blank=True, default='', max_length=11, verbose_name='手机号码')),
                ('address', models.CharField(blank=True, default='', max_length=255, verbose_name='收货地址')),
                ('pay_type', models.IntegerField(choices=[(0, '支付宝'), (1, '微信支付'), (2, '货到付款')], default=0, verbose_name='支付方式')),
                ('pay_sn', models.CharField(blank=True, default='', max_length=125, verbose_name='支付流水号')),
                ('pay_time', models.DateTimeField(blank=True, null=True, verbose_name='支付时间')),
                ('is_verify', models.BooleanField(default=False, verbose_name='是否已核销')),
                ('verify_time', models.DateTimeField(blank=True, null=True, verbose_name='核销时间')),
                ('is_comment', models.BooleanField(default=False, verbose_name='是否评价')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '订单',
                'verbose_name_plural': '订单',
                'ordering': ['-created_time'],
                'abstract': False,
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopSpec',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopspec', verbose_name='父级')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '规格模版',
                'verbose_name_plural': '规格模版',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopOrdersGoods',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('price', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='价格')),
                ('quantity', models.IntegerField(default=1, verbose_name='数量')),
                ('specs', models.JSONField(blank=True, default=list, help_text='规格数据', verbose_name='规格')),
                ('sku_sn', models.CharField(blank=True, default='', max_length=50, verbose_name='商品编码')),
                ('name', models.CharField(blank=True, default='', max_length=50, verbose_name='商品名称')),
                ('detail', models.TextField(blank=True, null=True, verbose_name='商品详情')),
                ('image', models.ImageField(blank=True, null=True, upload_to='goods', verbose_name='商品主图')),
                ('orders', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshoporders', verbose_name='订单')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('sku', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.baykeshopgoodssku', verbose_name='商品')),
            ],
            options={
                'verbose_name': '订单商品',
                'verbose_name_plural': '订单商品',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopOrdersComment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('content', models.TextField(verbose_name='评论内容')),
                ('reply_content', models.TextField(blank=True, help_text='回复内容', null=True, verbose_name='回复内容')),
                ('status', models.BooleanField(default=True, help_text='状态', verbose_name='状态')),
                ('score', models.IntegerField(choices=[(1, '⭐'), (2, '⭐⭐'), (3, '⭐⭐⭐'), (4, '⭐⭐⭐⭐'), (5, '⭐⭐⭐⭐⭐')], default=5, verbose_name='评分')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comments', to='shop.baykeshoporders', verbose_name='订单')),
                ('reply_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reply_comments', to=settings.AUTH_USER_MODEL, verbose_name='回复用户')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '订单评论',
                'verbose_name_plural': '订单评论',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopGoodsImages',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('image', models.ImageField(help_text='_(建议尺寸: 800*800，排在第一个的图片会作为商品主图)', upload_to='goods/images', verbose_name='商品图片')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('goods', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopgoods', verbose_name='商品')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '商品图片',
                'verbose_name_plural': '商品图片',
                'ordering': ['order'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeShopCarts',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('quantity', models.IntegerField(default=1, verbose_name='数量')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('sku', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shop.baykeshopgoodssku', verbose_name='商品')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='用户')),
            ],
            options={
                'verbose_name': '购物车',
                'verbose_name_plural': '购物车',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddConstraint(
            model_name='baykeshopordersgoods',
            constraint=models.UniqueConstraint(fields=('sku', 'orders'), name='sku_orders_unique'),
        ),
        migrations.AddConstraint(
            model_name='baykeshoporders',
            constraint=models.UniqueConstraint(fields=('user', 'order_sn'), name='unique_order_sn'),
        ),
        migrations.AddConstraint(
            model_name='baykeshopcarts',
            constraint=models.UniqueConstraint(fields=('user', 'sku'), name='unique_carts'),
        ),
    ]



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from baykeshop.db import BaseModel
from baykeshop.contrib.shop.models import BaykeShopOrders, BaykeShopOrdersGoods

User = get_user_model()

class BaykeShopOrdersComment(BaseModel):
    """
    订单评论
    """

    class ScoreChoices(models.IntegerChoices):
        """
        评分
        """
        ONE = 1, _('⭐')
        TWO = 2, _('⭐⭐')
        THREE = 3, _('⭐⭐⭐')
        FOUR = 4, _('⭐⭐⭐⭐')
        FIVE = 5, _('⭐⭐⭐⭐⭐')

    order = models.ForeignKey(
        BaykeShopOrders,
        verbose_name=_('订单'),
        on_delete=models.CASCADE,
        related_name='comments'
    )
    user = models.ForeignKey(
        User,
        verbose_name=_('用户'),
        on_delete=models.CASCADE,
    )
    content = models.TextField(
        verbose_name=_('评论内容'),
    )
    reply_user = models.ForeignKey(
        User,
        verbose_name=_('回复用户'),
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name='reply_comments'
    )
    reply_content = models.TextField(
        verbose_name=_('回复内容'),
        help_text=_('回复内容'),
        null=True,
        blank=True
    )
    status = models.BooleanField(
        verbose_name=_('状态'),
        default=True,
        help_text=_('状态')
    )
    score = models.IntegerField(
        verbose_name=_('评分'),
        choices=ScoreChoices.choices,
        default=ScoreChoices.FIVE
    )   

    class Meta:
        verbose_name = _('订单评论')
        verbose_name_plural = _('订单评论')
        ordering = ['-created_time']

    def __str__(self):
        return f'{self.order.order_sn}'
    
    @classmethod
    def get_spu_queryset(cls, spu):
        orders = BaykeShopOrdersGoods.objects.filter(sku__goods=spu).values_list('orders', flat=True).distinct()
        queryset = cls.objects.filter(order_id__in=orders, status=True)
        return queryset.order_by('-created_time')
    
    @classmethod
    def get_user_queryset(cls, user):
        queryset = cls.objects.filter(user=user)
        return queryset.order_by('-created_time')
    
    def save(self, *args, **kwargs):
        self.user = self.order.user
        return super().save(*args, **kwargs)
    
    def get_reply_user(self):
        if self.reply_user:
            return self.reply_user.username
        return ''
    
    def get_reply_content(self):
        if self.reply_content:
            return self.reply_content
        return ''
    
    # 获取平均分
    @classmethod
    def get_score_avg(cls, spu):
        score_avg = cls.get_spu_queryset(spu).aggregate(
            score_avg=models.Avg('score')
        ).get('score_avg') or 4.8
        return round(score_avg, 1)
    
    # 获取评论总数
    @classmethod
    def get_comment_count(cls, spu):
        return cls.get_spu_queryset(spu).count()
    
    # 获取好评率
    @classmethod
    def get_spu_comment_avg_score(cls, spu):
        """ 获取商品好评率  """
        gte_3 = cls.get_spu_queryset(spu).filter(score__gte=3).count()
        rate = gte_3 / cls.get_comment_count(spu) if cls.get_comment_count(spu) else 0.98
        return round(rate * 100, 1)
    
    @classmethod
    def get_user_comment_avg_score(cls, user):
        """ 获取用户好评率 """
        gte_3 = cls.get_user_queryset(user).filter(score__gte=3).count()
        rate = gte_3 / cls.get_user_queryset(user).count() if cls.get_user_queryset(user).count() else 0.98
        return round(rate * 100, 2)


from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ShopConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'baykeshop.contrib.shop'
    verbose_name = _('商品管理')
    verbose_name_plural = _('商品管理')

    def ready(self):
        from . import signals


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.utils import timezone

from .base import BaseModel


User = get_user_model()


class BaseCartsModel(BaseModel):
    """购物车"""
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('用户'))
    quantity = models.IntegerField(default=1, verbose_name=_('数量'))

    class Meta:
        abstract = True
    


from django import forms
from django.forms.utils import RenderableFormMixin


class BaseFormMixins:
    """ 表单样式混入 """
    template_name_bulma = 'baykeshop/forms/default.html'
    template_name_label = 'baykeshop/forms/label.html'

    def as_bulma(self):
        return self.render(self.template_name_bulma)


from django import forms
from django.contrib.admin.widgets import AdminTextareaWidget


class Form(forms.Form):
    pass


class ModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field in self.fields.values():
            if isinstance(field.widget, AdminTextareaWidget):
                field.widget.attrs['rows'] = 3



from django.contrib import messages
from django.utils.translation import gettext_lazy as _

from rest_framework import mixins, viewsets
from rest_framework import authentication
from rest_framework import permissions

from baykeshop.contrib.shop.models import BaykeShopOrders

from .serializers import BaykeShopOrdersCreateSerializer


class BaykeShopOrdersViewSet(mixins.CreateModelMixin,
                             mixins.DestroyModelMixin, 
                             viewsets.GenericViewSet):
    """ 创建订单 """
    queryset = BaykeShopOrders.objects.all()
    serializer_class = BaykeShopOrdersCreateSerializer
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    lookup_url_kwarg = 'order_sn'
    lookup_field = 'order_sn'
    
    def perform_destroy(self, instance):
        super().perform_destroy(instance)
        messages.success(self.request, _('订单删除成功'))




import re
import json

from django.db import models
from django.db.utils import OperationalError
from django.utils.translation import gettext_lazy as _
from baykeshop.contrib.system.validators import (
    validate_dict_value, is_bool, is_dict, is_json, is_list
)

from baykeshop.db import BaseModel


class BaykeDictModel(BaseModel):
    """ 字典管理  """
    name = models.CharField(max_length=50, verbose_name=_('名称'))
    key = models.SlugField(max_length=50, verbose_name=_('键'))
    value = models.TextField(
        verbose_name=_('值'), 
        help_text=_('''
            布尔值：true/false; 
            键值对：key1:value1一行一对;
            list:value1,value2一行一个值;
            json:{"key1":"value1"}
        ''' ), 
        validators=[validate_dict_value]
    )

    class Meta:
        verbose_name = _('字典管理')
        verbose_name_plural = verbose_name
        ordering = ['-created_time']
        constraints = [
            models.UniqueConstraint(fields=['key', 'site'], name='unique_key_site')
        ]

    def __str__(self):
        return self.name
    
    @classmethod
    def get_key_value(cls, key):
        """ 获取字典值 """
        try:
            obj = cls.current_site.get(key=key)
            if is_bool(obj.value):
                _v = json.loads(obj.value.lower())
                return _v
            if is_dict(obj.value):
                values = obj.value.splitlines()
                _v = {}
                for item in values:
                    k, v = item.split(':')
                    _v[k] = v
                return _v
            if is_list(obj.value) and obj.value.splitlines().__len__() == 1:
                return obj.value
            if is_list(obj.value) and obj.value.splitlines().__len__() > 1:
                return obj.value.splitlines()
            if is_json(obj.value):
                return json.loads(obj.value)
        except Exception:
            return None
        


from django.db.models import Sum, F, Q
from django.db.models.functions import TruncDate
from django.contrib.auth import get_user_model
from django.utils import timezone
from baykeshop.contrib.system.models import Visit
from baykeshop.contrib.shop.models import BaykeShopOrders

User = get_user_model()


class DateService:
    """分析服务"""

    @property
    def _today(self):
        """今天"""
        return timezone.now().date()

    @property
    def _yesterday(self):
        """昨天"""
        return self._today - timezone.timedelta(days=1)

    @property
    def _last_week(self):  # 上周
        """上周"""
        return self._today - timezone.timedelta(days=7)

    @property
    def _last_month(self):  # 上月
        """上月"""
        return self._today - timezone.timedelta(days=30)

    # 获取最近一周的日期
    def get_last_week_date(self):
        """获取最近一周的日期"""
        return [self._last_week + timezone.timedelta(days=i) for i in range(7)]

    def get_last_month_date(self):
        """获取最近一月的日期"""
        return [self._last_month + timezone.timedelta(days=i) for i in range(30)]


class AnalysisService(DateService):
    """分析服务"""

    date_field = "created_time"
    queryset = None

    def __init__(self, request=None):
        super().__init__()
        self.request = request

    @property
    def _date_field(self):  # 默认时间字段
        """时间字段"""
        return self.date_field

    def get_queryset(self):
        """获取查询集"""
        if self.queryset is None:  # 缓存
            return User.objects.none()
        return self.queryset.annotate(date=TruncDate(self._date_field))

    def count(self):
        """总数"""
        queryset = self.get_queryset()
        if queryset.exists():  # 存在则取最后一条
            return queryset.count()
        return 0

    def today_count(self):
        """今日总数"""
        queryset = self.get_queryset().filter(date=self._today)
        if queryset.exists():  # 存在则取最后一条
            return queryset.count()
        return 0

    def yesterday_count(self):
        """昨日总数"""
        queryset = self.get_queryset().filter(date=self._yesterday)
        if queryset.exists():  # 存在则取最后一条
            return queryset.count()
        return 0

    def last_week_count(self):
        """最近一周总数"""
        queryset = self.get_queryset().filter(
            date__range=(self._last_week, self._today)
        )
        if queryset.exists():
            return queryset.count()
        return 0

    def last_month_count(self):
        """最近一月总数"""
        queryset = self.get_queryset().filter(
            date__range=(self._last_month, self._today)
        )
        if queryset.exists():
            return queryset.count()
        return 0

    def get_last_week_data(self):
        """最近一周数据"""
        return [
            self.get_queryset().filter(date=date).count()
            for date in self.get_last_week_date()
        ]

    def get_last_month_data(self):
        """最近一月数据"""
        return [
            self.get_queryset().filter(date=date).count()
            for date in self.get_last_month_date()
        ]

    def get_last_week_data_dict(self, format=None):
        """最近一周数据字典
        @param format: 日期格式 %Y-%m-%d
        """
        if format:  # 获取日期字符串
            return {
                date.strftime(format): count
                for date, count in zip(
                    self.get_last_week_date(), self.get_last_week_data()
                )
            }
        return dict(zip(self.get_last_week_date(), self.get_last_week_data()))

    def get_last_month_data_dict(self, format=None):
        """最近一月数据字典"""
        if format:  # 获取日期字符串
            return {
                date.strftime(format): count
                for date, count in zip(
                    self.get_last_month_date(), self.get_last_month_data()
                )
            }
        return dict(zip(self.get_last_month_date(), self.get_last_month_data()))

    # 日环比率
    def get_ratio(self):
        """日环比率"""
        if self.yesterday_count() == 0:
            return 0
        return round(
            (self.today_count() - self.yesterday_count())
            / self.yesterday_count()
            * 100,
            2,
        )

    def get_data(self, format=None):
        """获取数据"""
        return {
            "count": self.count(),
            "today_count": self.today_count(),
            "yesterday_count": self.yesterday_count(),
            "last_week_count": self.last_week_count(),
            "last_month_count": self.last_month_count(),
            # 'last_week_data': self.get_last_week_data(),
            # 'last_month_data': self.get_last_month_data(),
            "last_week_data_dict": self.get_last_week_data_dict(format),
            "last_month_data_dict": self.get_last_month_data_dict(format),
            "ratio": self.get_ratio(),
        }


class UserAnalysisService(AnalysisService):
    """用户分析服务"""

    # queryset = User.objects.filter(is_staff=False)
    queryset = User.objects.all()
    date_field = "date_joined"


class OrderAnalysisService(AnalysisService):
    """订单分析服务"""

    def __init__(
        self,
        model=None,
        request=None,
    ):
        super().__init__(request)
        self.model = model or BaykeShopOrders
        self.queryset = self.model.objects.all()

    def get_queryset(self):
        """获取查询集"""
        return (
            super()
            .get_queryset()
            .exclude(
                Q(status=self.model.OrderStatus.UNPAID)
                | Q(status=self.model.OrderStatus.EXPIRED)
                | Q(status=self.model.OrderStatus.REFUNDED)
            )
        )

    # 昨日销售额
    def yesterday_sales(self):
        """昨日销售额"""
        return (
            self.get_queryset()
            .filter(date=self._yesterday)
            .aggregate(sales=Sum("pay_price"))["sales"]
            or 0
        )

    # 今日销售额
    def today_sales(self):
        """今日销售额"""
        return (
            self.get_queryset()
            .filter(date=self._today)
            .aggregate(sales=Sum("pay_price"))["sales"]
            or 0
        )

    # 本月销售额
    def last_month_sales(self):
        """本月销售额"""
        return (
            self.get_queryset()
            .filter(date__range=(self._last_month, self._today))
            .aggregate(sales=Sum("pay_price"))["sales"]
            or 0
        )

    def get_week_sales_data(self):
        """销售额数据"""
        return [
            self.get_queryset()
            .filter(date=date)
            .aggregate(sales=Sum("pay_price"))["sales"]
            or 0
            for date in self.get_last_week_date()
        ]

    def get_week_sales_data_dict(self, format=None):
        """销售额数据字典"""
        if format:  # 获取日期字符串
            return {
                date.strftime(format): count
                for date, count in zip(
                    self.get_last_week_date(), self.get_week_sales_data()
                )
            }
        return dict(zip(self.get_last_week_date(), self.get_week_sales_data()))

    def get_month_sales_data(self):
        """本月销售额数据"""
        return [
            self.get_queryset()
            .filter(date=date)
            .aggregate(sales=Sum("pay_price"))["sales"]
            or 0
            for date in self.get_last_month_date()
        ]

    def get_month_sales_data_dict(self, format=None):
        """本月销售额数据字典"""
        if format:  # 获取日期字符串
            return {
                date.strftime(format): count
                for date, count in zip(
                    self.get_last_month_date(), self.get_month_sales_data()
                )
            }
        return dict(zip(self.get_last_month_date(), self.get_month_sales_data()))

    def get_sales_ratio(self):
        """销售额环比率"""
        if self.yesterday_sales() == 0:
            return 0
        return round(
            (self.today_sales() - self.yesterday_sales())
            / self.yesterday_sales()
            * 100,
            2,
        )

    def get_sales_data(self, format=None):
        return {
            "yesterday_sales": self.yesterday_sales(),
            "today_sales": self.today_sales(),
            "last_month_sales": self.last_month_sales(),
            # 'week_sales_data': self.get_week_sales_data(),
            "week_sales_data_dict": self.get_week_sales_data_dict(format),
            # 'month_sales_data': self.get_month_sales_data(),
            "month_sales_data_dict": self.get_month_sales_data_dict(format),
            "sales_ratio": self.get_sales_ratio(),
        }


class VisitAnalysisService(AnalysisService):
    """访问分析服务"""

    def get_queryset(self):
        return Visit.objects.all()

    def _count(self):
        """总量计算"""
        datas = self.get_queryset().aggregate(count_uv=Sum("uv"), count_pv=Sum("pv"))
        return self._to_int_dict(datas)

    def count(self):
        """总访问量"""
        return self._count()

    def today_count(self):
        """今日访问量"""
        queryset = self.get_queryset().filter(date=self._today)
        datas = queryset.aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
        return self._to_int_dict(datas)

    def yesterday_count(self):
        """昨日访问量"""
        queryset = self.get_queryset().filter(date=self._yesterday)
        datas = queryset.aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
        return self._to_int_dict(datas)

    def last_week_count(self):  # 获取上周访问量
        """上周访问量"""
        queryset = self.get_queryset().filter(date__range=(self._last_week, self._yesterday))
        datas = queryset.aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
        return self._to_int_dict(datas)

    def last_month_count(self):  # 获取上月访问量
        """上月访问量"""
        queryset = self.get_queryset().filter(date__range=(self._last_month, self._yesterday))
        datas = queryset.aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
        return self._to_int_dict(datas)

    def get_last_week_data(self):
        """最近一周的pv及uv数据"""
        return [
            self._to_int_dict(
                self.get_queryset()
                .filter(date=date)
                .aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
            )
            for date in self.get_last_week_date()
        ]

    def get_last_month_data(self):
        """最近一月的pv及uv数据"""
        return [
            self._to_int_dict(
                self.get_queryset()
                .filter(date=date)
                .aggregate(count_pv=Sum("pv"), count_uv=Sum("uv"))
            )
            for date in self.get_last_month_date()
        ]

    def _uv_ration(self, field_name="count_uv"):
        if self.yesterday_count()[field_name] == 0:
            return 0
        ration = round(
            (self.today_count()[field_name] - self.yesterday_count()[field_name])
            / self.yesterday_count()[field_name]
            * 100,
            2,
        )
        return ration

    def _pv_ration(self, field_name="count_pv"):
        if self.yesterday_count()[field_name] == 0:
            return 0
        ration = round(
            (self.today_count()[field_name] - self.yesterday_count()[field_name])
            / self.yesterday_count()[field_name]
            * 100,
            2,
        )
        return ration

    def get_ratio(self):
        """获取比率"""
        return {"pv": self._pv_ration(), "uv": self._uv_ration()}

    # None转换为0
    def _to_int(self, value):
        return value if value else 0

    # 字典中value为None转换为0
    def _to_int_dict(self, data):
        return {k: self._to_int(v) for k, v in data.items()}



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
# Register your models here.
from baykeshop.sites import admin as bayke_admin
from .models import BaykeArticleContent, BaykeArticleCategory, BaykeArticleTags, BaykeSidebar


class BaykeArticleCategoryInline(bayke_admin.TabularInline):
    extra = 1
    model = BaykeArticleCategory
    verbose_name = '子分类'
    verbose_name_plural = '子分类'


@admin.register(BaykeArticleCategory)
class BaykeArticleCategoryAdmin(bayke_admin.ModelAdmin):
    """ 文章分类管理器 """
    list_display = ('id', 'name', 'is_show', 'order', 'created_time')
    list_display_links = ('id', 'name')
    list_filter = ('is_show',)
    search_fields = ('name',)
    ordering = ('-order',)
    readonly_fields = ('parent',)
    list_editable = ('is_show', 'order')
    inlines = [BaykeArticleCategoryInline]


@admin.register(BaykeArticleContent)
class BaykeArticleContentAdmin(bayke_admin.ModelAdmin):
    """ 文章内容管理器 """
    list_display = ('id', 'title', 'category', 'user', 'order', 'created_time')
    list_display_links = ('id', 'title')
    search_fields = ('title',)
    ordering = ('-created_time',)
    readonly_fields = ('user', )
    filter_horizontal = ('tags',)
    fieldsets = (
        (_('文章信息'), {
            'fields': ('title', 'category', 'tags', )
        }),
        (_('文章内容'), {
            'fields': ('content', 'user', 'order')
        }),
    )

    def save_model(self, request, obj, form, change):
        obj.user = request.user
        return super().save_model(request, obj, form, change)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'category':
            kwargs['queryset'] = BaykeArticleCategory.objects.filter(parent__isnull=False)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)
    

@admin.register(BaykeArticleTags)
class BaykeArticleTagsAdmin(bayke_admin.ModelAdmin):
    """ 文章标签管理器 """
    list_display = ('id', 'name', 'created_time')
    list_display_links = ('id', 'name')
    search_fields = ('name',)
    ordering = ('-created_time',)


@admin.register(BaykeSidebar)
class BaykeSidebarAdmin(bayke_admin.ModelAdmin):
    """ 侧边栏管理器 """
    list_display = ('id', 'name', 'icon', 'module', 'order', 'is_show', 'created_time')
    list_display_links = ('id', 'name')
    search_fields = ('name',)
    ordering = ('-order',)
    list_filter = ('module',)
    list_editable =('order', 'is_show')


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :echarts.py
@说明    :分析图表
@时间    :2024/12/06 22:47:24
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from django.db import models
from django.utils.translation import gettext_lazy as _

from pyecharts.charts import Bar, Line, Pie
from pyecharts import options as opts
from pyecharts.globals import ThemeType

from baykeshop.db.analysis import (
    OrderAnalysisService,
    VisitAnalysisService,
    UserAnalysisService,
)

order_analysis_service = OrderAnalysisService()
visit_analysis_service = VisitAnalysisService()
user_analysis_service = UserAnalysisService()


def orders_chart():
    """订单统计图表"""
    data = order_analysis_service.get_month_sales_data_dict(format="%m-%d")
    x_data = list(data.keys())
    y_data = list(data.values())
    month_counts = order_analysis_service.get_last_month_data()
    colors = ["#79AEC8", "#d14a61", "#675bba"]
    bar = (
        Bar(
            init_opts=opts.InitOpts(
                width="calc(100% - 290px);",
                height="350px",
                theme=ThemeType.WALDEN,
                js_host="/static/baykeshop/js/",
                # bg_color="#F8F8F8",
            )
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title="订单统计", pos_left="30"),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            toolbox_opts=opts.ToolboxOpts(is_show=True),
        )
        .add_xaxis(xaxis_data=x_data)
        .add_yaxis(
            series_name="订单金额", y_axis=y_data, yaxis_index=1, color=colors[0]
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                name="金额",
                type_="value",
                min_=0,
                max_=max(y_data),
                position="right",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color=colors[1])
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value}"),
            )
        )
        .extend_axis(
            yaxis=opts.AxisOpts(
                type_="value",
                name="数量",
                min_=0,
                max_=max(month_counts),
                position="left",
                axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color=colors[2])
                ),
                axislabel_opts=opts.LabelOpts(formatter="{value} 单"),
                splitline_opts=opts.SplitLineOpts(
                    is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
                ),
            )
        )
    )

    line = (
        Line()
        .add_xaxis(xaxis_data=x_data)
        .add_yaxis(
            series_name="订单数", y_axis=month_counts, yaxis_index=2, color=colors[2]
        )
    )
    return bar.overlap(line).render_embed()


def users_chart():
    """用户统计图表"""
    datas = user_analysis_service.get_last_week_data_dict("%m-%d")
    x_data = list(datas.keys())
    y_data = list(datas.values())
    c = (
        Line(
            init_opts=opts.InitOpts(
                width="calc(100% - 100px);",
                # width="100%",
                height="300px",
                theme=ThemeType.WALDEN,
                js_host="/static/baykeshop/js/",
                # bg_color="#F8F8F8",
            )
        )
        .add_xaxis(x_data)
        .add_yaxis("", y_data, is_smooth=True)
        .set_series_opts(
            areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
            label_opts=opts.LabelOpts(is_show=False),
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title="用户统计", pos_left="30"),
            xaxis_opts=opts.AxisOpts(
                axistick_opts=opts.AxisTickOpts(is_align_with_label=True),
                is_scale=False,
                boundary_gap=False,
            ),
        )
        .render_embed()
    )
    return c


def user_pie_chart():
    """用户统计图表"""
    # 本月新增用户数
    user_datas = user_analysis_service.get_queryset().filter(
        date__range=(user_analysis_service._last_month, user_analysis_service._today)
    )
    # 未消费用户数
    no_order_user_datas = user_datas.filter(baykeshoporders__isnull=True)
    # 消费1次以上用户数
    one_order_user_datas = (
        user_datas.filter(baykeshoporders__isnull=False)
        .annotate(count=models.Count("id"))
        .filter(count__gt=1)
    )
    # 排除本月新增的用户
    exclude_month_users = (
        user_analysis_service.get_queryset()
        .exclude(
            date__range=(
                user_analysis_service._last_month,
                user_analysis_service._today,
            )
        )
        .filter(baykeshoporders__isnull=False)
    )
    # 回流用户数,获取过去消费的用户数据和本月消费的用户数据做交集
    reflux_users = exclude_month_users.values_list("id", flat=True).intersection(
        one_order_user_datas.values_list("id", flat=True)
    )
    results = [
        ["留存", len(set(reflux_users))],
        ["消费1次以上", one_order_user_datas.count()],
        ["未消费", no_order_user_datas.count()],
        ["新增", user_analysis_service.last_month_count()],
    ]

    c = (
        Pie(
            init_opts=opts.InitOpts(
                width="600px",
                # width="100%",
                height="300px",
                theme=ThemeType.WALDEN,
                js_host="/static/baykeshop/js/",
                # bg_color="#F8F8F8",
            )
        )
        .add(
            "本月用户消费统计",
            results,
            center=["40%", "55%"],
        )
        .set_global_opts(
            title_opts=opts.TitleOpts(title=_("当月购买用户统计")),
            legend_opts=opts.LegendOpts(pos_left="70%"),
        )
        .set_series_opts(label_opts=opts.LabelOpts(formatter="{b}: {c}"))
    ).render_embed()
    return c



from django.views.generic import DetailView, View
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse_lazy, reverse
from django.http.response import HttpResponse, HttpResponseRedirect
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from django.utils.decorators import method_decorator

from alipay.aop.api.util.SignatureUtils import verify_with_rsa
from baykeshop.contrib.shop.models.orders import BaykeShopOrders
from baykeshop.contrib.system.models import BaykeDictModel


class BaykeShopOrdersPayView(LoginRequiredMixin, DetailView):
    context_object_name = "order"
    login_url = reverse_lazy("member:login")
    model = BaykeShopOrders
    slug_field = "order_sn"
    slug_url_kwarg = "order_sn"
    template_name = "baykeshop/shop/pay.html"

    def get_queryset(self):
        return BaykeShopOrders.objects.filter(user=self.request.user)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = _("订单支付")
        return context


class AlipayCallBackVerifySignMixin:
    """支付宝支付回调，验签"""

    def has_verify_sign(self, data):
        """验签
        data是从请求中获得的字典数据，携带 sign和sign_type
        """
        sign = data.pop("sign")
        sign_type = data.pop("sign_type")
        alipay_public_key = BaykeDictModel.get_key_value("ALIPAY_PUBLIC_KEY")
        # 去除sign和sign_type参数之后进行升序排列，拼装请求参数用支付宝公钥进行验签
        message = "&".join(
            [
                f"{k}={v}"
                for k, v in dict(
                    sorted(data.items(), key=lambda d: d[0], reverse=False)
                ).items()
            ]
        )
        flag = verify_with_rsa(
            alipay_public_key, message.encode("UTF-8", "strict"), sign
        )
        return flag


@method_decorator(csrf_exempt, name="dispatch")
class AlipayCallbackView(AlipayCallBackVerifySignMixin, View):
    """支付宝支付结果通知"""

    def get(self, request, *args, **kwargs):
        """支付宝同步通知"""
        data = request.GET.dict()
        success = self.has_verify_sign(data)
        order_sn = data.get("out_trade_no")
        if success:
            order = BaykeShopOrders.objects.filter(order_sn=order_sn).first()
            if order:
                order.pay_time = timezone.now()
                order.pay_sn = data.get("trade_no")
                order.status = BaykeShopOrders.OrderStatus.PAID
                order.save()
            return HttpResponseRedirect(
                reverse("member:orders-detail", kwargs={"order_sn": order_sn})
            )
        return HttpResponse("success")

    def post(self, request, *args, **kwargs):
        """支付宝异步通知"""
        data = request.POST.dict()
        order_sn = data.get("out_trade_no")
        success = self.has_verify_sign(data)
        if success:
            order = BaykeShopOrders.objects.filter(order_sn=order_sn).first()
            if order:
                order.pay_time = timezone.now()
                order.pay_sn = data.get("trade_no")
                order.status = BaykeShopOrders.OrderStatus.PAID
                order.save()
            return HttpResponse("success")
        


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from baykeshop.db import validators
from .base import BaseModel


User = get_user_model()


class BaseUserModel(BaseModel):
    """用户基础信息"""

    class GenderChoices(models.TextChoices):
        MALE = "male", _("男")
        FEMALE = "female", _("女")
        UNKNOWN = "unknown", _("未知")

    nickname = models.CharField(
        max_length=50, blank=True, null=True, verbose_name=_("昵称")
    )
    avatar = models.ImageField(
        upload_to="avatar", blank=True, null=True, verbose_name=_("头像")
    )
    gender = models.CharField(
        max_length=10,
        choices=GenderChoices.choices,
        default=GenderChoices.MALE,
        verbose_name=_("性别"),
    )
    birthday = models.DateField(blank=True, null=True, verbose_name=_("生日"))
    mobile = models.CharField(
        max_length=11,
        blank=True,
        null=True,
        verbose_name=_("手机号码"),
        unique=True,
        validators=[validators.validate_phone],
    )
    qq = models.CharField(max_length=20, blank=True, null=True, verbose_name=_("QQ"))
    wechat = models.CharField(
        max_length=50, blank=True, null=True, verbose_name=_("微信")
    )
    description = models.TextField(blank=True, null=True, verbose_name=_("简介"))

    class Meta:
        verbose_name = _("用户信息")
        verbose_name_plural = _("用户信息")
        ordering = ["-created_time"]
        abstract = True


class BaseUserAddressModel(BaseModel):
    """用户地址"""

    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_("用户"))
    name = models.CharField(max_length=50, verbose_name=_("收货人"))
    province = models.CharField(max_length=50, verbose_name=_("省"))
    city = models.CharField(max_length=50, verbose_name=_("市"))
    district = models.CharField(max_length=50, verbose_name=_("区"))
    address = models.CharField(max_length=255, verbose_name=_("详细地址"))
    phone = models.CharField(max_length=50, verbose_name=_("手机"))
    is_default = models.BooleanField(default=False, verbose_name=_("是否默认"))

    class Meta:
        verbose_name = _("地址")
        verbose_name_plural = verbose_name
        ordering = ["-is_default"]
        abstract = True



from django.core import management
from django.core.management.base import BaseCommand
from baykeshop.conf import bayke_settings


class Command(BaseCommand):
    help = "初始化项目数据"

    def handle(self, *args, **options):
        site_id = bayke_settings.SITE_ID
        management.call_command(
            "add_dict", 
            site_id, 'SITE_TITLE', "站点名称", 
            bayke_settings.SITE_TITLE
        )
        management.call_command(
            "add_dict", 
            site_id, 'SITE_HEADER', "管理后台使用的站点名称", 
            bayke_settings.SITE_HEADER
        )
        management.call_command(
            "add_dict", 
            site_id, 'INDEX_TITLE', "管理后台首页标题", 
            bayke_settings.INDEX_TITLE
        )
        management.call_command(
            "add_dict", 
            site_id, 'SITE_DESCRIPTION', "站点描述", 
            bayke_settings.DESCRIPTION
        )
        management.call_command(
            "add_dict", 
            site_id, 'SITE_KEYWORDS', "站点关键字", 
            bayke_settings.KEYWORDS
        )
        management.call_command(
            "add_dict", 
            site_id, 'COPYRIGHT', "版权信息", 
            bayke_settings.COPYRIGHT
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_HOST', "SMTP 服务器地址", 
            bayke_settings.EMAIL_HOST
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_PORT', "SMTP 服务器端口", 
            bayke_settings.EMAIL_PORT
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_HOST_USER', "SMTP 服务器用户名", 
            bayke_settings.EMAIL_HOST_USER
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_HOST_PASSWORD', "SMTP 服务器密码", 
            bayke_settings.EMAIL_HOST_PASSWORD
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_USE_TLS', "SMTP 服务器是否使用 TLS", 
            bayke_settings.EMAIL_USE_TLS
        )
        management.call_command(
            "add_dict", 
            site_id, 'EMAIL_USE_SSL', "SMTP 服务器是否使用 SSL", 
            bayke_settings.EMAIL_USE_SSL
        )
        management.call_command(
            "add_dict", 
            site_id, 'ALIPAY_APPID', "支付宝 APPID", 
            bayke_settings.ALIPAY_APPID
        )
        management.call_command(
            "add_dict", 
            site_id, 'ALIPAY_PUBLIC_KEY', "支付宝公钥", 
            bayke_settings.ALIPAY_PUBLIC_KEY
        )
        management.call_command(
            "add_dict", 
            site_id, 'ALIPAY_PRIVATE_KEY', "支付宝私钥", 
            bayke_settings.ALIPAY_PRIVATE_KEY
        )
        management.call_command(
            "add_dict", 
            site_id, 'ICP', "备案号", 
            bayke_settings.ICP
        )




from .auth import (
    BaykeShopUserLoginView, BaykeShopUserLogoutView, BaykeShopUserRegisterView
)
from .profile import (
    BaykeShopUserProfileView, BaykeShopUserAddressListView, BaykeShopUserPasswordView,
    BaykeShopUserAddressCreateView, BaykeShopUserAddressUpdateView, BaykeShopUserAddressDeleteView,
    BaykeShopUserProfileUpdateView
)
from .orders import BaykeShopOrdersListView, BaykeShopOrdersDetailView
from .actions import (
    OrderStatusActionView, CommentActionView
)


import json
from django import forms
from django.utils.translation import gettext_lazy as _
from django.contrib.admin import widgets
from baykeshop.forms import ModelForm
from .models import BaykeShopGoodsSKU, BaykeShopSpec


class MyFilteredSelectMultiple(widgets.FilteredSelectMultiple):
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        if value is not None:
            if isinstance(value, str):
                try:
                    value = json.loads(value)
                    value = [int(v['id']) for v in value]
                except (json.JSONDecodeError, ValueError):
                    value = []
            elif isinstance(value, list):
                value = [int(v) for v in value]
            else:
                value = [v.pk for v in value]
        context['widget']['value'] = value
        return context


class BaykeShopGoodsSKUForm(ModelForm):
    """商品SKU表单"""
    specs = forms.ModelMultipleChoiceField(
        queryset=BaykeShopSpec.get_queryset().filter(parent__isnull=False),
        label=_('商品规格'),
        widget=MyFilteredSelectMultiple(_('商品规格'), False),
        required=False,
        help_text=_('单规格时可以不选； 多规格时必选且同一个规格下只能选一个规格值'),
    )

    class Meta:
        model = BaykeShopGoodsSKU
        fields = '__all__'
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 初始化规格值回显
        if self.instance and self.instance.specs:
            try:
                specs_ids = [spec['id'] for spec in json.loads(self.instance.specs)]
                self.initial['specs'] = BaykeShopSpec.get_queryset().filter(id__in=specs_ids)
            except (json.JSONDecodeError, ValueError):
                self.initial['specs'] = []

    def clean_specs(self):
        specs = self.cleaned_data.get('specs')
        if specs.exists():
            values = specs.values('id', 'parent__name', 'name')
            # 父级名称
            parent_names = [
                spec['parent__name'] for spec in values
            ]
            if specs.count() > len(set(parent_names)):
                raise forms.ValidationError(_('同一个规格下只能选一个规格值'))
            
            json_data = json.dumps(list(values), ensure_ascii=False)
            specs = json_data
            return specs
        return json.dumps([], ensure_ascii=False)
        



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :views.py
@说明    :购物车模块视图类
@时间    :2024/12/01 09:29:59
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets, mixins
from rest_framework.pagination import PageNumberPagination
from rest_framework.authentication import SessionAuthentication
from rest_framework.permissions import IsAuthenticated


from baykeshop.contrib.shop.models import BaykeShopCarts
from .serializers import BaykeShopCartsSerializer


class BaykeShopCartsViewSet(mixins.ListModelMixin, 
                            mixins.CreateModelMixin,
                            mixins.UpdateModelMixin,
                            mixins.DestroyModelMixin, 
                            viewsets.GenericViewSet):
    """购物车模块视图类"""
    pagination_class = PageNumberPagination
    authentication_classes = [SessionAuthentication, ]
    permission_classes = [IsAuthenticated]
    serializer_class = BaykeShopCartsSerializer

    def get_queryset(self):
        return BaykeShopCarts.objects.filter(user=self.request.user)
    
    def perform_create(self, serializer):
        serializer.save()
        messages.success(self.request, _('添加购物车成功'))

    def perform_destroy(self, instance):
        instance.delete()
        messages.success(self.request, _('删除购物车成功'))




from django.urls import path, re_path

from . import views

app_name = 'shop'

urlpatterns = [
    # 支付宝支付回调(必须在第一个)
    path('pay/callback/', views.AlipayCallbackView.as_view(), name='alipay-callback'),   
    # 商品列表
    path('list/', views.BaykeShopGoodsListView.as_view(), name='list'),
    # 搜索
    path('search/', views.BaykeShopSearchView.as_view(), name='search'),
    # 商品详情
    path('detail/<int:pk>/', views.BaykeShopGoodsDetailView.as_view(), name='detail'),
    # 商品分类
    path('category/<int:pk>/', views.BaykeShopCategoryListView.as_view(), name='category'),
    # 购物车
    path('carts/', views.BaykeShopCartsListView.as_view(), name='carts'),
    # 商品结算页面
    re_path(r'^list/(?P<skuid>\d+)/(?P<num>\d+)/$', views.BaykeShopCashView.as_view(), name='cash-sku'),
    # 购物车结算页面
    re_path(r'^carts/(?P<skuids>\d+(?:,\d+)*)/$', views.BaykeShopCashView.as_view(), name='cash_cartids'),
    # 支付页面
    path('pay/<slug:order_sn>/', views.BaykeShopOrdersPayView.as_view(), name='orders-pay'),
    # 首页
    path('', views.BaykeShopIndexView.as_view(), name='index'),
]


from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

class SystemConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'baykeshop.contrib.system'
    verbose_name = _('系统管理')



from .base import BaseModel
from .user import BaseUserModel, BaseUserAddressModel
from .orders import BaseOrdersModel, BaseOrdersGoodsModel
from .goods import BaseGoodsModel, BaseGoodsSKUModel
from .category import BaseCategoryModel
from .carts import BaseCartsModel
from .article import BaseArticleModel




from django.core import management
from django.core.management.base import BaseCommand
from django.utils.translation import gettext_lazy as _
from django.contrib.sites.models import Site

from baykeshop.contrib.system.models import BaykeDictModel


class Command(BaseCommand):
    help = _('Create a dictionary')

    def add_arguments(self, parser):
        parser.add_argument('site', type=int)   # site_id
        parser.add_argument('key', type=str)    # 键
        parser.add_argument('name', type=str)   # 名称
        parser.add_argument('value', type=str)  # 值

    def handle(self, *args, **options):
        site = self.create_site(options['site'])
        key = options['key']
        name = options['name']
        value = options['value']
        obj, iscreated = BaykeDictModel.objects.update_or_create(
            site=site, 
            key=key, 
            defaults={
                'name': name, 
                'value': value,
                'site': site, 
                'key': key, 
            }
        )
        _type = _('添加') if iscreated else _('更新')
        self.stdout.write(self.style.SUCCESS(_(f'{obj.name} { _type } 成功')))
        
    def create_site(self, site_id):
        """创建站点"""
        site = Site.objects.filter(id=site_id).first()
        if site:
            return site
        return Site.objects.create(name='bayke', domain='bayke.site')


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :serializers.py
@说明    :购物车模块序列化类
@时间    :2024/12/01 09:26:32
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.db import models
from django.db.utils import IntegrityError
from django.contrib.auth import get_user_model
from django.contrib import messages
from django.utils.translation import gettext_lazy as _

from rest_framework import serializers

from baykeshop.contrib.shop.models import BaykeShopCarts


User = get_user_model()


class BaykeShopCartsSerializer(serializers.ModelSerializer):
    """ 加入购物车序列化类 """
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = BaykeShopCarts
        fields = ('user', 'sku', 'quantity')
        validators = []

    def create(self, validated_data):
        # 判断是否已经在购物车，如果在，则仅更新数量
        try:
            instance = super().create(validated_data)
        except IntegrityError:
            queryset = BaykeShopCarts.objects.filter(
                user=validated_data['user'],
                sku=validated_data['sku']
            )
            queryset.update(quantity=models.F('quantity') + validated_data['quantity'])
            instance = queryset.first()
        return instance
    
    def update(self, instance, validated_data):
        instance.quantity = validated_data['quantity']
        instance.save()
        messages.success(self.context['request'], _('更新购物车数量成功'))
        return instance



from django.template import Library

from baykeshop.contrib.article.models import BaykeSidebar


register = Library()

@register.simple_tag
def sidebars():
    """ 侧边栏 """
    return BaykeSidebar.objects.filter(is_show=True).order_by('order')


from django.db import models
from django.db.models.functions import Coalesce
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core.cache import cache
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from baykeshop.db import BaseModel


class VisitManager(models.Manager):
    def create_pv_uv(self, request, content_object: models.Model):
        """创建PV和UV数据"""
        ip = self.get_client_ip(request)
        date = timezone.now().date()
        content_type = ContentType.objects.get_for_model(content_object)
        try:
            obj = self.get(
                ip=ip, date=date, 
                content_type=content_type, 
                object_id=content_object.id
            )
            if cache.get(f'{ip}-{date}-{content_object.pk}') is None:
                # 第二次创建缓存1小时
                cache.set(f'{ip}-{date}-{content_object.pk}', True, 60 * 60)
                obj.pv = models.F('pv') + 1
                obj.save()
                
        except self.model.DoesNotExist:
            obj = self.create(
                ip=ip, date=date, 
                content_type=content_type, 
                object_id=content_object.id,
                pv=1,
                uv=1
            )
            # 第一次创建缓存10分钟
            cache.set(f'{ip}-{date}-{content_object.pk}', True, 60 * 10)
        return obj
    
    def get_uv_pv_count(self, content_object: models.Model):
        """获取UV数量"""
        content_type = ContentType.objects.get_for_model(content_object)
        object_id = content_object.id
        queryset = self.filter(content_type=content_type, object_id=object_id)
        count_aggregate = queryset.aggregate(
            uv=Coalesce(models.Sum('uv'), 0),
            pv=Coalesce(models.Sum('pv'), 0)
        )
        return count_aggregate
    
    @staticmethod
    def get_client_ip(request):
        """获取客户端IP地址"""
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip
    

class Visit(BaseModel):
    """分析数据模型"""
    content_type = models.ForeignKey(
        ContentType, 
        on_delete=models.CASCADE, 
        verbose_name=_('内容类型')
    )
    object_id = models.PositiveBigIntegerField(verbose_name=_('对象ID'))
    content_object = GenericForeignKey('content_type', 'object_id')
    ip = models.GenericIPAddressField(verbose_name=_('IP'), default='127.0.0.1')
    pv = models.PositiveBigIntegerField(verbose_name=_('PV'))
    uv = models.PositiveBigIntegerField(verbose_name=_('UV'))
    date = models.DateField(verbose_name=_('日期'))

    objects = VisitManager()

    class Meta:
        verbose_name = _('分析数据')
        verbose_name_plural = verbose_name
        ordering = ['-date']
        indexes = [
            models.Index(fields=['date']),
            models.Index(fields=['content_type', 'object_id']),
            models.Index(fields=['content_type', 'object_id', 'date']),
        ]        

    def __str__(self):
        return f"{self.content_object} / {self.date} / {self.pv} / {self.uv}"